/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ClockListener.java 1083 2006-09-14 14:28:22Z ofabre $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.engine.clock;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.common.xmlmanipulation.SourceHelper;

/**
 * @author alouis
 */
public class ClockListener implements Runnable {

    private DeliveryChannel channel;

    private MessageExchange messageExchange;

    private boolean running;

    private Logger logger;

    public ClockListener(DeliveryChannel channel, Logger logger) {
        super();
        this.channel = channel;
        this.logger = logger;
    }

    public void run() {

        running = true;

        mainLoop: while (running) {
            try {
                messageExchange = channel.accept();
                if (messageExchange != null) {
                    process(messageExchange);
                }
            } catch (MessagingException e) {
                logger.log(Level.SEVERE, e.getMessage(), e);
                continue mainLoop;
            }
        }

    }

    private void process(MessageExchange msg) {
        logger.log(Level.INFO, "");
        try {
            // receive an acquittement ?
            if (ExchangeStatus.DONE.equals(msg.getStatus())) {
                logger.log(Level.INFO, "receive a status DONE. nothing to do.");
                return;
            } else if (new QName("time").equals(msg.getOperation())
                    && msg instanceof InOut) {
                logger.log(Level.INFO,
                        "The ClockImpl service is called to return current time");

                DateFormat formatter = new SimpleDateFormat(
                        "MM/dd/yyyy HH:mm:ss");
                String en = "<timeResponse xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" "
                        + "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"><timeReturn xsi:type=\"xsd:string\">";
                en += formatter.format(new Date(System.currentTimeMillis()));
                en += "</timeReturn></timeResponse>";

                logger.log(Level.INFO, "The ClockImpl answer : " + en);

                NormalizedMessage nm = msg.createMessage();

                nm.setContent(SourceHelper.createSource(en));
                msg.setMessage(nm, "OUT");
            } else {
                Exception e = new Exception(
                        "only 'time'/InOut operation is recognized");
                msg.setError(e);
                msg.setStatus(ExchangeStatus.ERROR);
                logger.log(Level.WARNING, e.getMessage(), e);
            }
            // Send the message (response and/or status)
            channel.send(msg);
        } catch (Throwable e) {
            logger.log(Level.SEVERE, e.getMessage(), e);
        }
    }

    public void stopProcessing() {
        this.running = false;
    }

}
