/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ClockServiceUnitManager.java 896 2006-08-03 10:58:08Z alouis $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.engine.clock;

import java.io.File;
import java.io.FileInputStream;

import javax.jbi.component.ServiceUnitManager;
import javax.jbi.management.DeploymentException;

import org.objectweb.petals.component.common.xmlmanipulation.ManagementMessageUtil;

/**
 * @author alouis
 */
public class ClockServiceUnitManager implements ServiceUnitManager {

    private ClockImpl clock;
    
    public ClockServiceUnitManager(ClockImpl clock) {
        this.clock = clock;
    }

    public String deploy(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        try {
            FileInputStream in = new FileInputStream(serviceUnitRootPath
                    + File.separator + serviceUnitName);
            TimeZonesMap tzm = new TimeZonesMap(in);
            // TODO check for duplicate
            clock.addTimeZonesForArtifact(serviceUnitName, tzm);
        } catch (Exception e) {
            throw new DeploymentException("Problem while deploying artefact "
                    + serviceUnitName, e);
        }
        return ManagementMessageUtil.getComponentTaskResult(clock.getContext()
                .getComponentName(), "deploy",
                ManagementMessageUtil.TASK_RESULT_SUCCESS);
    }

    public void init(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        // no specific init
    }

    public void shutDown(String serviceUnitName) throws DeploymentException {
    }

    public void start(String serviceUnitName) throws DeploymentException {

    }

    public void stop(String serviceUnitName) throws DeploymentException {
    }

    public String undeploy(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        clock.removeTimeZonesForArtifact(serviceUnitName);
        return null;
    }

}
