/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : ${file_name} 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.filetransferbc.listeners;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOptionalOut;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.RobustInOnly;

import org.objectweb.petals.binding.filetransferbc.FileTransferBCException;
import org.objectweb.petals.component.common.handleMessagesExchange.HandleResponses;
import org.objectweb.petals.component.common.xmlmanipulation.SourceHelper;
import org.objectweb.petals.component.common.xmlmanipulation.StringHelper;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author alouis, Rocio BARRAZA - eBMWebsourcing
 * 
 */
public class FileTransferBCJBIListener implements Runnable {

    protected String responseDirName;

    private DeliveryChannel channel;

    private Logger logger;

    private boolean running;

    public FileTransferBCJBIListener(DeliveryChannel channel, Logger logger,
        ComponentContext context, String dir) {
        this.channel = channel;
        this.logger = logger;
        this.responseDirName = dir + "download" + File.separator;
    }

    public boolean handleInMessage(MessageExchange msg)
        throws FileTransferBCException {
        // Read the IN message
        String content = null;
        NormalizedMessage nm = msg.getMessage("IN");
        try {
            content = SourceHelper.createString(nm.getContent());
        } catch (Exception e) {
            throw new FileTransferBCException(e.getMessage());
        }

        String dirDest = msg.getEndpoint().getEndpointName();
        if (StringHelper.isEmpty(content)) {
            logger.log(Level.INFO, "content is null");
        } else {

            writeFile(content, dirDest, "content.xml");
        }
        Set attachId = nm.getAttachmentNames();
        try {
            if (!attachId.isEmpty()) {
                writeAttachments(attachId, nm, dirDest);
            }
        } catch (FileTransferBCException ex) {
            throw new FileTransferBCException(ex.getMessage());
        }
        return true;
    }

    public void process(MessageExchange messageExchange) {
        try {
            if (messageExchange != null) {
                if (!(messageExchange.getStatus().equals(ExchangeStatus.DONE))) {
                    {
                        if (messageExchange.getStatus().equals(
                            ExchangeStatus.ERROR)) {
                            try {
                                String dirDest = messageExchange.getEndpoint()
                                    .getEndpointName();
                                writeFile(SourceHelper
                                    .createStringContent(messageExchange
                                        .getFault().getContent()), dirDest,
                                    "log.txt");
                                messageExchange.setStatus(ExchangeStatus.DONE);
                            } catch (Exception e) {
                                this.logger.log(Level.WARNING,
                                    "Error getting error message");
                            }
                        } else if (messageExchange instanceof InOnly
                            && !(messageExchange instanceof RobustInOnly)) {
                            // Handle InOnly message
                            processInOnlyMessages(messageExchange);
                        } else if ((messageExchange instanceof InOptionalOut)
                            || (messageExchange instanceof RobustInOnly)) {

                            processRobustInOnlyMessages(messageExchange);
                        } else if (messageExchange instanceof InOut) {
                            // Handle InOut message
                            try {
                                HandleResponses
                                    .faultMessageExchange(messageExchange,
                                        new FileTransferBCException(
                                            "InOut exchanges don't support"),
                                        "505");
                            } catch (JBIException ex) {
                                this.logger.log(Level.WARNING, ex.getMessage());
                            }

                        }
                        channel.send(messageExchange);
                    }
                } else {
                    this.logger.log(Level.INFO, "Receive state DONE");
                }
            }
        } catch (MessagingException e) {
            logger.log(Level.SEVERE, e.getMessage(), e);
        }

    }

    public void run() {

        running = true;
        MessageExchange messageExchange = null;
        while (running) {
            try {
                messageExchange = channel.accept();
                process(messageExchange);
            } catch (MessagingException e) {
                running = false;
                logger.log(Level.SEVERE, e.getMessage(), e);
            }
        }
    }

    public void stopProcessing() {
        this.running = false;
    }

    public boolean writeAttachments(Set attachId, NormalizedMessage nm,
        String subDir) throws FileTransferBCException {
        Set<DataHandler> attachHandler = new HashSet<DataHandler>();
        for (Object object : attachId) {
            attachHandler.add(nm.getAttachment((String) object));
        }

        for (DataHandler handler : attachHandler) {
            try {
                File file = new File(this.responseDirName + File.separator
                    + subDir);
                if (!file.exists())
                    file.mkdirs();

                FileOutputStream dos = null;
                File fileTemp = new File(file.getAbsolutePath()
                    + File.separator + handler.getName());
                if (fileTemp.exists()) {
                    GregorianCalendar allDate = new GregorianCalendar();

                    String date = allDate.get(Calendar.DAY_OF_MONTH) + " / "
                        + (allDate.get(Calendar.MONTH) + 1) + " / "
                        + allDate.get(Calendar.YEAR) + "/"
                        + allDate.get(Calendar.HOUR) + ":"
                        + allDate.get(Calendar.MINUTE)
                        + allDate.get(Calendar.SECOND);

                    dos = new FileOutputStream(new File(file.getAbsolutePath()
                        + File.separator + handler.getName() + date));
                } else {
                    dos = new FileOutputStream(new File(file.getAbsolutePath()
                        + File.separator + handler.getName()));
                }
                handler.writeTo(dos);
                dos.flush();
                dos.close();

            } catch (Exception e) {
                throw new FileTransferBCException("Error: writing attachment "
                    + handler.getName() + "  " + e.getMessage());
            }
        }
        return true;
    }

    public boolean writeFile(String body, String subDir, String fileName)
        throws FileTransferBCException {
        try {
            File file = new File(this.responseDirName + subDir);
            if (!file.exists()) {
                file.mkdirs();
            }

            BufferedWriter out = null;
            File fileTemp = new File(file.getAbsolutePath() + File.separator
                + fileName);
            if (fileTemp.exists()) {
                GregorianCalendar allDate = new GregorianCalendar();

                String date = allDate.get(Calendar.DAY_OF_MONTH) + "-"
                    + (allDate.get(Calendar.MONTH) + 1) + "-"
                    + allDate.get(Calendar.YEAR) + "-"
                    + allDate.get(Calendar.HOUR) + ":"
                    + allDate.get(Calendar.MINUTE)
                    + allDate.get(Calendar.SECOND);

                out = new BufferedWriter(new FileWriter(fileTemp
                    .getAbsolutePath()
                    + date));
            } else {
                out = new BufferedWriter(new FileWriter(fileTemp
                    .getAbsolutePath()));
            }
            out.write(body);
            out.close();
            return true;
        } catch (IOException e) {
            throw new FileTransferBCException("Error creating new file "
                + fileName, e.getMessage());
        }
    }

    protected void processInOnlyMessages(MessageExchange msg) {

        try {
            handleInMessage(msg);
        } catch (FileTransferBCException ex) {
            this.logger.log(Level.WARNING, "InOnly " + ex.getMessage());
        }
        try {
            msg.setStatus(ExchangeStatus.DONE);
        } catch (MessagingException e) {
            this.logger.log(Level.WARNING,
                "Error: Can not set message status to DONE");
        }
    }

    protected void processRobustInOnlyMessages(MessageExchange msg) {

        try {
            handleInMessage(msg);
            try {
                msg.setStatus(ExchangeStatus.DONE);
            } catch (MessagingException e) {
                this.logger.log(Level.WARNING,
                    "Error: Can not set message status to DONE");
            }
        } catch (FileTransferBCException ex) {
            try {
                HandleResponses.faultMessageExchange(msg, ex, "505");
            } catch (JBIException ex2) {
                this.logger.log(Level.WARNING, ex2.getMessage());
            }
        }

    }
}
