/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : ${file_name} 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.filetransferbc.listeners;

import java.io.File;
import java.util.HashSet;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.RobustInOnly;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.objectweb.petals.binding.filetransferbc.FileTransferBCException;
import org.objectweb.petals.component.common.handleMessagesExchange.HandleResponses;
import org.objectweb.petals.component.common.xmlmanipulation.SourceHelper;
import org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper;
import org.w3c.dom.Document;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Rocio BARRAZA - eBMWebsourcing
 * 
 */
public class FileTransferBCListener implements Runnable {

    protected Set<DataHandler> attachments;

    protected FileTransferBCJBIListener listenerJBI;

    private ComponentContext context;

    private DeliveryChannel channel;

    private String directoryRoot;

    private Logger logger;

    private Map<String, Properties> mapDirConfig;

    private boolean running;

    public FileTransferBCListener(Logger logger, DeliveryChannel channel,
        ComponentContext context, FileTransferBCJBIListener listenerJBI,
        Map<String, Properties> mapDirConfig, String dir) {
        super();
        this.context = context;
        this.channel = channel;
        this.mapDirConfig = mapDirConfig;
        this.listenerJBI = listenerJBI;
        this.logger = logger;
        this.directoryRoot = dir + File.separator + "upload" + File.separator;
        attachments = new HashSet<DataHandler>();
    }

    public File createWorkDirectory() {

        File file = null;
        file = new File(directoryRoot);
        if (!file.exists()) {
            file.mkdirs();
        }
        File filewaste = new File(directoryRoot + "waste");
        if (!filewaste.exists()) {
            filewaste.mkdirs();
        }
        return file;
    }

    public boolean handleInOnlyMessage(String body, QName service,
        String operation) throws FileTransferBCException {

        try {
            InOnly msg = null;
            msg = channel.createExchangeFactory().createInOnlyExchange();
            // Create and send messageExchange
            HandleResponses.buildInMessage(msg, body, service, operation,
                this.attachments);
            channel.sendSync(msg);
            if (ExchangeStatus.DONE.equals(msg.getStatus())) {
                return true;
            } else {
                return false;
            }

        } catch (Exception ex) {
            throw new FileTransferBCException(ex.getMessage());
        }
    }

    public boolean handleInOutMessage(String body, QName service,
        String operation, String dirDest) throws FileTransferBCException {
        InOut msg = null;
        String response = "";
        String fileName = "";
        try {

            msg = channel.createExchangeFactory().createInOutExchange();

            HandleResponses.buildInMessage(msg, body, service, operation,
                this.attachments);
            channel.sendSync(msg);

            // Verify if request was success,if not, handle the error message
            if (msg.getStatus().equals(ExchangeStatus.ERROR)) {
                fileName = "fault";
                response = SourceHelper.createString(msg.getFault()
                    .getContent());
            } else {
                fileName = "content";
                // if there wasn't errors , binding component handle out reponse
                NormalizedMessage nm = msg.getMessage("OUT");
                response = SourceHelper.createString(nm.getContent());
                this.listenerJBI.writeAttachments(nm.getAttachmentNames(), nm,
                    dirDest);
            }
            this.listenerJBI.writeFile(response, dirDest, fileName);
            // Send status DONE to end exchange
            msg.setStatus(ExchangeStatus.DONE);
            channel.send(msg);
            return true;

        } catch (Exception ex) {
            throw new FileTransferBCException(ex.getMessage());
        }
    }

    public void handleNewFiles(File file, boolean inOut, QName service,
        String operation, String dirDest) {

        boolean writen = false;
        File[] filesSecond = file.listFiles();
        String body = "<?xml version='1.0' encoding='UTF-8'?><info>attached file(s)</info>";
        for (File file2 : filesSecond) {
            if (!file2.isDirectory() && service != null) {
                DataHandler dhFile = new DataHandler(new FileDataSource(file2));
                attachments.add(dhFile);
            }
        }
        try {
            if (!inOut) {
                writen = handleInOnlyMessage(body, service, operation);
            } else {
                writen = handleInOutMessage(body, service, operation, dirDest);
            }
        } catch (FileTransferBCException e) {
            this.logger.log(Level.WARNING, e.getMessage());
            moveFilesToWaste(filesSecond, file.getParentFile()
                .getAbsolutePath());
        }
        if (writen) {
            moveFilesToWaste(filesSecond, file.getParentFile()
                .getAbsolutePath());
        }
    }

    /**
     * Handle RobustInOnly messages
     * 
     * @param mailCorps
     * @param service
     * @param operation
     * @throws SMTPBCException
     */
    public void handleRobustInOnlyMessage(String body, QName service,
        String operation, String dirDest) throws FileTransferBCException {

        try {
            RobustInOnly msg = null;
            msg = channel.createExchangeFactory().createRobustInOnlyExchange();

            HandleResponses.buildInMessage(msg, body, service, operation,
                this.attachments);
            channel.sendSync(msg);

            if (msg.getStatus().equals(ExchangeStatus.ERROR)) {
                this.listenerJBI.writeFile(SourceHelper.createString(msg
                    .getFault().getContent()), dirDest, "fault");
                msg.setStatus(ExchangeStatus.DONE);
                channel.send(msg);
            }

        } catch (Exception ex) {
            throw new FileTransferBCException(ex.getMessage());
        }
    }

    public void notFoundService(String response, String dirDest) {

        try {
            this.listenerJBI.writeFile(response, dirDest, "log.txt");
        } catch (FileTransferBCException e) {
            this.logger.log(Level.INFO, "Error writing log file", e
                .getMessage());
        }
    }

    public void run() {

        running = true;

        File file = createWorkDirectory();
        while (running) {
            File[] files = file.listFiles();
            for (File fileTemp : files) {
                if (!fileTemp.getName().equals("waste")
                    && fileTemp.isDirectory()) {
                    Properties props = mapDirConfig.get(fileTemp.getName());
                    if (props != null && (fileTemp.listFiles().length > 0)) {
                        process(fileTemp);
                    }
                }
            }
            try {
                Thread.sleep(2000);
            } catch (InterruptedException e) {
                this.logger.log(Level.WARNING, e.getMessage());
            }
        }

    }

    public void stopProcessing() {
        running = false;
    }

    protected void moveFilesToWaste(File[] files, String parent) {

        String str = parent + File.separator + "waste" + File.separator;
        for (File file2 : files) {
            file2.renameTo(new File(str + file2.getName()));
        }
        attachments.clear();
    }

    protected void process(File currentDir) {

        boolean found = false;
        boolean inOut = false;

        Properties props = this.mapDirConfig.get(currentDir.getName());
        String service = props.getProperty("service.name");

        ServiceEndpoint[] servs = this.context.getEndpointsForService(QName
            .valueOf(service));

        String operation = props.getProperty("operation.name");
        String dirDest = props.getProperty("directory.destination")
            + File.separator;
        ServiceEndpoint endpoint = null;
        if (servs.length > 0) {
            endpoint = servs[0];
            Document serviceDesc;
            try {
                serviceDesc = context.getEndpointDescriptor(endpoint);

                // Determines if the operation exists and if it is an inOnly or
                // inOut operation
                try {
                    found = WSDLHelper.hasOperationNamed(serviceDesc,
                        operation, QName.valueOf(service));
                    inOut = WSDLHelper.isInOutOperation(serviceDesc, operation,
                        QName.valueOf(service));
                } catch (Exception e) {
                    /*
                     * 
                     */
                    found = false;
                    inOut = false;
                    this.logger.log(Level.WARNING, e.getMessage());
                }

            } catch (JBIException e) {
                this.logger.log(Level.INFO,
                    "Error verifying operation definition ", e.getMessage());
            }
        }
        if (!found) {
            String response = SourceHelper.createSoapFault(
                new FileTransferBCException("Service unknown"), "300");
            notFoundService(response, dirDest);
        }
        // Handle each directory
        handleNewFiles(currentDir, inOut, endpoint.getServiceName(), operation,
            dirDest);

    }

}
