/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : ${file_name} 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.filetransferbc;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

import java.lang.reflect.Method;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.binding.filetransferbc.FileTransferBC;
import org.objectweb.petals.binding.filetransferbc.listeners.FileTransferBCJBIListener;
import org.objectweb.petals.binding.filetransferbc.listeners.FileTransferBCListener;
import org.objectweb.petals.binding.filetransferbc.mock.MockDeliveryChannel;
import org.objectweb.petals.binding.filetransferbc.mock.MockFileTransferBC;
import org.objectweb.petals.binding.filetransferbc.mock.MockServiceEndpoint;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.w3c.dom.Document;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Rocio BARRAZA - eBMWebsourcing
 * 
 */
public class FileTranferBCTest extends TestCase {

    Logger logger;

    @Override
    public void setUp() throws Exception {
        logger = EasyMock.createMock(Logger.class);
    }

    public void testGetExtensionMBeanName() {
        // Run Test Case
        FileTransferBC FileTransferBC = new FileTransferBC();
        ObjectName objectName = FileTransferBC.getExtensionMBeanName();
        assertNull(objectName);
    }

    public void testGetServiceDescription() {
        // Create mock object and set suManager in a new method

        // Create Mock Objects
        PetalsServiceUnitManager suManager = createMock(PetalsServiceUnitManager.class);
        MockFileTransferBC fileTransferBC = new MockFileTransferBC();

        // Create test parameters
        Document document = null;
        ServiceEndpoint fooService = createMock(ServiceEndpoint.class);
        // new InternalEndpoint(new QName(
        // "fooService"), "fooServiceEndpoint", null, null, null);
        ServiceEndpoint barService = createMock(ServiceEndpoint.class);
        // new InternalEndpoint(new QName(
        // "barService"), "barServiceEndpoint", null, null, null);
        try {
            document = DocumentBuilderFactory.newInstance()
                .newDocumentBuilder().newDocument();
            expect(suManager.getServiceDescription(fooService)).andReturn(
                document).anyTimes();
            expect(suManager.getServiceDescription(barService)).andReturn(null)
                .anyTimes();
            expect(suManager.getServiceDescription(null)).andReturn(null)
                .anyTimes();
        } catch (ParserConfigurationException e2) {
            e2.printStackTrace();
        }
        replay(suManager);

        fileTransferBC.setServiceUnitManager(suManager);
        // Run the test case 1
        // Asking service description for foo service
        Document doc = null;
        doc = fileTransferBC.getServiceDescription(fooService);
        assertEquals(document, doc);

        // Run the test case 2
        // Asking service description for bar service
        doc = null;
        doc = fileTransferBC.getServiceDescription(barService);
        assertNull("This service description should be null", doc);

        // Run the test case 3
        // Asking service description for null service
        doc = null;
        doc = fileTransferBC.getServiceDescription(null);
        assertNull("This service description should be null", doc);

        verify(suManager);
    }

    public void testInit() {

        // Create Mock Objects
        Method method1 = null;
        Method method4 = null;
        Method method5 = null;
        Method method6 = null;
        ComponentContext componentContext = null;
        try {

            method1 = FileTransferBC.class.getDeclaredMethod(
                "createServiceUnitManager", new Class[] {});
            method4 = FileTransferBC.class.getDeclaredMethod(
                "createFileTransferBCJBIListener", new Class[] {});
            method5 = FileTransferBC.class.getDeclaredMethod(
                "createFileTransferBCListener", new Class[] {});
            method6 = FileTransferBC.class.getDeclaredMethod(
                "getDirectoryRoot", new Class[] {});

        } catch (SecurityException e1) {
            e1.printStackTrace();
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        }
        Method[] mockedMethods = new Method[] {method1, method5, method4,
            method6};

        FileTransferBC FileTransferBC = createMock(FileTransferBC.class,
            mockedMethods);
        componentContext = createMock(ComponentContext.class);
        PetalsServiceUnitManager suManager = createMock(PetalsServiceUnitManager.class);
        FileTransferBCJBIListener jbilist = createMock(FileTransferBCJBIListener.class);
        FileTransferBCListener list = createMock(FileTransferBCListener.class);

        // Create test parameters
        Logger logger = Logger.getLogger("", null);
        MockDeliveryChannel channel = new MockDeliveryChannel();

        // Init mocks
        try {
            expect(componentContext.getDeliveryChannel()).andReturn(channel);
            expect(componentContext.getComponentName()).andReturn("test bc");
            expect(componentContext.getLogger("", null)).andReturn(logger);
            expect(FileTransferBC.createServiceUnitManager()).andReturn(
                suManager);
            expect(FileTransferBC.createFileTransferBCJBIListener()).andReturn(
                jbilist);
            expect(FileTransferBC.createFileTransferBCListener()).andReturn(
                list);
            expect(FileTransferBC.getDirectoryRoot())
                .andReturn("directoryRoot");
        } catch (JBIException e) {
            e.printStackTrace();
        }
        replay(componentContext);
        replay(FileTransferBC);

        // Run the Test
        try {
            FileTransferBC.init(componentContext);
        } catch (JBIException e) {

        }
        verify(componentContext);
        verify(FileTransferBC);

    }

    public void testIsExchangeWithConsumerOkay() {
        // Create Mock Objects
        MessageExchange msgExchange = createMock(MessageExchange.class);

        FileTransferBC fileTransferBC = new FileTransferBC();

        // Create test parameters
        ServiceEndpoint se = new MockServiceEndpoint();

        // Run the Test
        boolean result = fileTransferBC.isExchangeWithConsumerOkay(se,
            msgExchange);
        assertEquals(result, true);
    }

    public void testIsExchangeWithProviderOkay() {
        // Create Mock Objects
        MessageExchange msgExchange = createMock(MessageExchange.class);

        FileTransferBC fileTransferBC = new FileTransferBC();

        // Create test parameters
        ServiceEndpoint se = new MockServiceEndpoint();

        // Run the Test
        boolean result = fileTransferBC.isExchangeWithProviderOkay(se,
            msgExchange);
        assertEquals(result, true);
    }

}