/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JmsBC.java 676 2006-06-27 15:44:03Z alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.jms;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.management.DeploymentException;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import org.objectweb.petals.binding.jms.incoming.IncomingJmsServiceUnitHandler;
import org.objectweb.petals.binding.jms.outgoing.JBIExchangeProcessor;
import org.objectweb.petals.binding.jms.outgoing.JmsConnector;
import org.objectweb.petals.binding.jms.outgoing.OutgoingJmsServiceUnitHandler;
import org.objectweb.petals.component.common.listener.MessageExchangeListener;
import org.objectweb.petals.component.common.logging.ComponentLogger;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.objectweb.petals.component.common.xmlmanipulation.XMLHelper;

import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Node;

/**
 * This component is a JMS BC
 * 
 * @author alouis - eBMWebsourcing
 */
public class JmsBC implements Component, ComponentLifeCycle {

    protected ComponentContext context;

    protected DeliveryChannel channel;

    protected MessageExchangeListener jbiListener;

    protected PetalsServiceUnitManager suManager;

    protected OutgoingJmsServiceUnitHandler outgoingJmsSUHandler;

    protected IncomingJmsServiceUnitHandler incomingJmsSUHandler;

    protected ComponentLogger logger;

    public ObjectName getExtensionMBeanName() {
        return null;
    }

    public ComponentLifeCycle getLifeCycle() {
        return this;
    }

    public Document getServiceDescription(ServiceEndpoint arg0) {
        return suManager.getServiceDescription(arg0);
    }

    public ServiceUnitManager getServiceUnitManager() {
        return this.suManager;
    }

    public void init(ComponentContext ctx) throws JBIException {

        Logger javaLogger = ctx.getLogger("", null);
        logger = new ComponentLogger(javaLogger, javaLogger.getName(),
            javaLogger.getResourceBundleName(), ctx.getComponentName());

        logger.log(Level.INFO, "init");

        this.context = ctx;

        channel = context.getDeliveryChannel();

        suManager = createServiceUnitManager();

        outgoingJmsSUHandler = new OutgoingJmsServiceUnitHandler(context,
            logger);

        suManager.addNewHandler(outgoingJmsSUHandler);

        incomingJmsSUHandler = new IncomingJmsServiceUnitHandler(context,
            channel, logger);

        suManager.addNewHandler(incomingJmsSUHandler);

    }

    public boolean isExchangeWithConsumerOkay(ServiceEndpoint arg0,
        MessageExchange arg1) {
        return true;
    }

    public boolean isExchangeWithProviderOkay(ServiceEndpoint arg0,
        MessageExchange arg1) {
        return true;
    }

    public ServiceEndpoint resolveEndpointReference(DocumentFragment arg0) {
        ServiceEndpoint result = null;

        if (arg0 != null) {
            String prefix = arg0.getPrefix();
            if (prefix == null) {
                prefix = new String();
            } else {
                prefix += ":";
            }
            Node service = XMLHelper.findChild(arg0, prefix + "service", true);

            if (service != null) {
                String serviceRequested = XMLHelper.getAttributeValue(service,
                    "name");
                if (serviceRequested != null) {
                    result = suManager.findEndpointForService(QName
                        .valueOf(serviceRequested));
                }
            }
        }
        return result;
    }

    public void shutDown() throws JBIException {

        logger.log(Level.INFO, "shutdown");

        channel.close();

        channel = null;

        jbiListener = null;
    }

    public void start() throws JBIException {
        // TODO process ERROR status ?
        logger.log(Level.INFO, "start");

        JmsConnector connector = new JmsConnector(outgoingJmsSUHandler, logger);

        JBIExchangeProcessor processor = new JBIExchangeProcessor(context,
            channel, suManager, connector, logger);

        jbiListener = new MessageExchangeListener(channel, processor,
            MessageExchangeListener.IgnoredStatus.DONE_AND_ERROR_IGNORED, 0,
            context.getComponentName());

        jbiListener.listen();

    }

    public void stop() throws JBIException {

        logger.log(Level.INFO, "stop");

        this.jbiListener.terminate();
    }

    // //////////////////////////////////////////////////////////////
    // implementation specific
    // //////////////////////////////////////////////////////////////

    protected PetalsServiceUnitManager createServiceUnitManager()
        throws DeploymentException {
        return new PetalsServiceUnitManager(context, logger);
    }
}
