/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JmsConnexion.java 676 2006-06-27 15:44:03Z alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.jms.su;

import javax.jms.Queue;
import javax.jms.QueueConnectionFactory;
import javax.jms.Topic;
import javax.jms.TopicConnectionFactory;
import javax.naming.Context;
import javax.naming.NamingException;

/**
 * This class is used to store information about an external JMS queue
 * acting as an external service
 * 
 * @version $Revision: $ $Date: $
 * @since Petals 1.0
 * @author alouis
 */
public class JmsConnection {

    protected Context context;

    protected String name, tcfName, qcfName;

    protected TopicConnectionFactory tcf;

    protected QueueConnectionFactory qcf;

    protected Queue queue;

    protected Topic topic;

    protected boolean topicConnection;

    public JmsConnection(Context ctx, String name, String tcf, String qcf) {
        context = ctx;
        this.name = name;
        this.tcfName = tcf;
        this.qcfName = qcf;

        topicConnection = (tcfName != null);
    }

    /**
     * Find in the JNDI directory the Topic and TopicConnFact,
     * or the Queue and QueueConnFact. 
     * @throws Exception
     */
    public void init() throws Exception {
        if (isTopicConnection()) {
            // init for a topic
            topic = (Topic) context.lookup(name);
            tcf = (TopicConnectionFactory) context.lookup(tcfName);
            context.close();
            if (tcf == null || topic == null) {
                throw new NamingException("Topic '" + name + "' or TCF '"
                    + tcfName + "' not found in JNDI.");
            }
        } else {
            // init for a queue
            queue = (Queue) context.lookup(name);
            qcf = (QueueConnectionFactory) context.lookup(qcfName);
            context.close();
            if (qcf == null || queue == null) {
                throw new NamingException("Queue '" + name + "' or QCF '"
                    + qcfName + "' not found in JNDI.");
            }
        }
    }

    public QueueConnectionFactory getQcf() {
        return qcf;
    }

    public TopicConnectionFactory getTcf() {
        return tcf;
    }

    public Queue getQueue() {
        return queue;
    }

    public Topic getTopic() {
        return topic;
    }

    public boolean isTopicConnection() {
        return topicConnection;
    }

    public boolean isQueueConnection() {
        return !topicConnection;
    }

}
