/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : ${file_name} 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.smtpbc;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.Properties;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.binding.exception.SMTPBCException;
import org.objectweb.petals.binding.smtpbc.SMTPBC;
import org.objectweb.petals.binding.smtpbc.listeners.SMTPBCJBIListener;
import org.objectweb.petals.binding.smtpbc.listeners.SMTPBCListener;
import org.objectweb.petals.binding.smtpbc.mock.MockSMTPBC;
import org.objectweb.petals.binding.smtpbc.mock.MockServiceEndpoint;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.w3c.dom.Document;

import junit.framework.TestCase;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Rocio BARRAZA - eBMWebsourcing
 * 
 */
public class SMTPBCTest extends TestCase {

    Logger logger;

    private String baseDir;

    @Override
    public void setUp() throws Exception {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        logger = EasyMock.createMock(Logger.class);
    }

    public void testGetExtensionMBeanName() {
        // Run Test Case
        SMTPBC smtpbc = new SMTPBC();
        ObjectName objectName = smtpbc.getExtensionMBeanName();
        assertNull(objectName);
    }

    public void testGetProperties() {
        MockSMTPBC smtpbc = new MockSMTPBC();
        ComponentContext componentContext = null;

        componentContext = createMock(ComponentContext.class);

        String install = baseDir
            + "src" + File.separator + "test" + File.separator + "files";

        expect(componentContext.getInstallRoot()).andReturn(install).anyTimes();

        replay(componentContext);

        Logger logger = Logger.getLogger("", null);
        smtpbc.setComponentContext(componentContext);
        smtpbc.setLogger(logger);
        Properties returnProps = null;
        try {
            returnProps = smtpbc.getProperties();
        } catch (SMTPBCException e) {
            e.printStackTrace();
        }

        Properties props = new Properties();

        try {
            props.load(new FileInputStream(baseDir
                + "src" + File.separator + "test" + File.separator + "files"
                + File.separator + "META-INF" + File.separator
                + "connection.properties"));
        } catch (FileNotFoundException e1) {
            e1.printStackTrace();
        } catch (IOException e1) {
            e1.printStackTrace();
        }
        assertEquals(props, returnProps);
        verify(componentContext);
    }

    public void testGetServiceDescription() {
        // Create mock object and set suManager in a new method

        // Create Mock Objects
        PetalsServiceUnitManager suManager = createMock(PetalsServiceUnitManager.class);
        MockSMTPBC smtpbc = new MockSMTPBC();

        // Create test parameters
        Document document = null;
        ServiceEndpoint fooService = new MockServiceEndpoint();
        ServiceEndpoint barService = new MockServiceEndpoint();
        try {
            document = DocumentBuilderFactory.newInstance()
                .newDocumentBuilder().newDocument();
            expect(suManager.getServiceDescription(fooService)).andReturn(
                document).anyTimes();
            expect(suManager.getServiceDescription(barService)).andReturn(null)
                .anyTimes();
            expect(suManager.getServiceDescription(null)).andReturn(null)
                .anyTimes();
        } catch (ParserConfigurationException e2) {
            e2.printStackTrace();
        }
        replay(suManager);

        smtpbc.setServiceUnitManager(suManager);
        // Run the test case 1
        // Asking service description for foo service
        Document doc = null;
        doc = smtpbc.getServiceDescription(fooService);
        assertEquals(document, doc);

        // Run the test case 2
        // Asking service description for bar service
        doc = null;
        doc = smtpbc.getServiceDescription(barService);
        assertNull("This service description should be null", doc);

        // Run the test case 3
        // Asking service description for null service
        doc = null;
        doc = smtpbc.getServiceDescription(null);
        assertNull("This service description should be null", doc);

        verify(suManager);
    }

    public void testInit() {

        // Create Mock Objects
        Method method4 = null;
        Method method5 = null;
        Method method6 = null;
        Method method7 = null;
        ComponentContext componentContext = null;
        try {
            method4 = SMTPBC.class.getDeclaredMethod("getProperties",
                new Class[] {});

            method5 = SMTPBC.class.getDeclaredMethod("createSMTPBCJBIListener",
                new Class[] {Properties.class});
            method6 = SMTPBC.class.getDeclaredMethod("createSMTPBCListener",
                new Class[] {Properties.class});
            method7 = SMTPBC.class.getDeclaredMethod(
                "createServiceUnitManager", new Class[] {});

        } catch (SecurityException e1) {
            e1.printStackTrace();
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        }
        Method[] mockedMethods = new Method[] {method7, method5, method6,
            method4};

        SMTPBC smtpbc = createMock(SMTPBC.class, mockedMethods);
        componentContext = createMock(ComponentContext.class);
        PetalsServiceUnitManager suManager = createMock(PetalsServiceUnitManager.class);
        SMTPBCJBIListener jbilist = createMock(SMTPBCJBIListener.class);
        SMTPBCListener list = createMock(SMTPBCListener.class);

        Properties props = new Properties();
        try {
            props.load(new FileInputStream(baseDir
                + "src" + File.separator + "test" + File.separator + "files"
                + File.separator + "META-INF" + File.separator
                + "connection.properties"));
        } catch (FileNotFoundException e1) {
            e1.printStackTrace();
        } catch (IOException e1) {
            e1.printStackTrace();
        }

        // Create test parameters
        Logger logger = Logger.getLogger("", null);
        // Init mocks
        try {
            expect(componentContext.getDeliveryChannel()).andReturn(null);
            expect(componentContext.getComponentName()).andReturn("test");
            expect(componentContext.getLogger("", null)).andReturn(logger);
            expect(smtpbc.createServiceUnitManager()).andReturn(suManager);
            try {
                expect(smtpbc.getProperties()).andReturn(props);
            } catch (SMTPBCException e) {
                e.printStackTrace();
            }
            expect(smtpbc.createSMTPBCJBIListener(props)).andReturn(jbilist);
            expect(smtpbc.createSMTPBCListener(props)).andReturn(list);
        } catch (JBIException e) {
            e.printStackTrace();
        }
        replay(componentContext);
        replay(smtpbc);

        // Run the Test
        try {
            smtpbc.init(componentContext);
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in SMTPBC initialisation");
        }
        verify(smtpbc);
        verify(componentContext);
    }

    public void testIsExchangeWithConsumerOkay() {
        // Create Mock Objects
        MessageExchange msgExchange = createMock(MessageExchange.class);

        SMTPBC smtpbc = new SMTPBC();

        // Create test parameters
        ServiceEndpoint se = new MockServiceEndpoint();

        // Run the Test
        boolean result = smtpbc.isExchangeWithConsumerOkay(se, msgExchange);
        assertEquals(result, true);
    }

    public void testIsExchangeWithProviderOkay() {
        // Create Mock Objects
        MessageExchange msgExchange = createMock(MessageExchange.class);

        SMTPBC smtpbc = new SMTPBC();

        // Create test parameters
        ServiceEndpoint se = new MockServiceEndpoint();

        // Run the Test
        boolean result = smtpbc.isExchangeWithProviderOkay(se, msgExchange);
        assertEquals(result, true);
    }

}
