/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: HelloworldProcessor.java 12:18:02 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.xslt;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.objectweb.petals.component.common.listener.AbstractInternalMEProcessor;
import org.objectweb.petals.component.common.xmlmanipulation.SourceHelper;
import org.objectweb.petals.component.common.xmlmanipulation.StringHelper;
import org.objectweb.petals.engine.xslt.exception.XsltException;
import org.xml.sax.InputSource;
import org.xml.sax.XMLFilter;
import org.xml.sax.XMLReader;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since   Petals 1.0
 * @author Rocio BARRAZA - eBMWebsourcing
 *
 */
public class XsltProcessor extends AbstractInternalMEProcessor {

    private Map<String, String> mapEndpointXslt;
    public XsltProcessor(DeliveryChannel c, Logger l, Map<String, String> mapEndpointXslt) {
        super(c, l);
        this.mapEndpointXslt=mapEndpointXslt;
    }

    @Override
    protected void sendInMessage(QName service, QName operation,
        NormalizedMessage in) throws XsltException{
        throw new XsltException(
        "Only InOut messages are supported");
    }

    @Override
    protected boolean sendInOutMessage(QName service, QName operation,
        NormalizedMessage in, NormalizedMessage out, boolean optionalOut)
        throws Exception {
            Source content =in.getContent();
            String contentString = SourceHelper.createString(content);
            
            if (!StringHelper.isEmpty(contentString)) {
                try{
               String fileXslt= getXsltFile(service.getLocalPart());
                    
               SAXParserFactory spf = SAXParserFactory.newInstance();
               
               spf.setNamespaceAware(true);
               XMLReader reader = spf.newSAXParser().getXMLReader();
               //Load xslt sheet to tranform xml 
               SAXTransformerFactory stf = (SAXTransformerFactory)TransformerFactory.newInstance();
               XMLFilter filtre1 = stf.newXMLFilter(new StreamSource(fileXslt));
               //Match reader with filters
               filtre1.setParent(reader);
               /*
                *Creation of source: the last filter, this filter indicate transformations to do. 
                * Transform NormalizeMessage content in SAXsource.
                */
               byte[] msg = contentString.getBytes();
               InputSource inSource = new InputSource(new ByteArrayInputStream(msg));
               SAXSource source = new SAXSource(filtre1, inSource);
               
               ByteArrayOutputStream outStream= new ByteArrayOutputStream();
               StreamResult resultat = new StreamResult(outStream);
               //Transformation 
               Transformer transformer = stf.newTransformer();
               transformer.transform(source , resultat);
               
               /*
                *Transform StreamResult to String , to be send in NormalizeMessage 
                */
               String str= new String(outStream.toByteArray());
               if(str != null && !("".equals(str))){
                   out.setContent(SourceHelper.createSource(str));
               }else{
                   throw new Exception(
                   "Error in xslt transformation");
               }
                }catch(Exception e){
                    throw new Exception(e.getMessage());
                }
               return true;
            }else{
                l.log(Level.WARNING, "");
                // this will be automaticly added to a Fault if the message
                throw new Exception(
                    "operation : message empty");
            }
    }
        
     public String getXsltFile(String serviceName) throws Exception{
         
         String xslt = null;
         
         xslt=this.mapEndpointXslt.get(serviceName);
         if(xslt != null){
             return xslt;
         }else{
             throw new Exception("Xslt sheet not found");
         }
     }

}
