/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): Adrien LOUIS
 * --------------------------------------------------------------------------
 * $Id: SourceHelper.java 94 2006-03-26 17:11:05Z alouis $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.util;

import java.io.ByteArrayInputStream;
import java.io.StringWriter;
import java.util.Properties;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

/**
 * Sample class used to manage the mapping between
 * <code>javax.xml.transform.Source</code> from String. the mapping is a
 * quicly "string" -> "<content>string</content>" conversion.
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * @author wjoseph - eBMWebSourcing
 */
public final class SourceHelper {

    private static Transformer transformer;

    private SourceHelper() {
        // avoid to be built
    }

    public static Source createSource(String msg) {
        StreamSource source = new StreamSource();
        byte[] msgByte = msg.getBytes();
        ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
        source.setInputStream(in);
        return source;
    }

    public static String createString(Source s) throws Exception {
        if (s instanceof StreamSource) {
            ((StreamSource) s).getInputStream().reset();
        }
        StringWriter buffer = new StringWriter();
        Result result = new StreamResult(buffer);
        Transformer transform = getTransformer();
        transform.transform(s, result);
        return buffer.toString();
    }

    protected static Transformer getTransformer()
        throws TransformerConfigurationException,
        TransformerFactoryConfigurationError {
        if (transformer == null) {
            transformer = TransformerFactory.newInstance().newTransformer();
            Properties props = new Properties();
            props.put(OutputKeys.OMIT_XML_DECLARATION, "yes");
            transformer.setOutputProperties(props);
        }
        return transformer;
    }

}
