/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: AbstractJBIPackage.java 114 2006-03-23 11:33:07Z alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.packaging;

import java.net.URI;

import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * Abstract JBI Package implementation.
 * 
 * @version $Rev: 114 $ $Date: 2006-03-23 12:33:07 +0100 (jeu., 23 mars 2006) $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
abstract class AbstractJBIPackage implements JBIPackage {

    protected LoggingUtil log;

    /**
     * Package's JBI Descriptor.
     */
    private JBIDescriptor descriptor;

    /**
     * The proper package location as URI.
     */
    private URI packageLocation;

    /**
     * Constructor used by implementations.
     */
    protected AbstractJBIPackage(URI packageZipUri, JBIDescriptor descriptor,
            Logger logger) {

        super();

        log = new LoggingUtil(logger);

        if (packageZipUri == null) {
            throw new IllegalArgumentException(
                    "Invalid null package Zip archive URI.");

        } else if (descriptor == null) {
            throw new IllegalArgumentException("Invalid null descriptor.");
        }

        this.packageLocation = packageZipUri;
        this.descriptor = descriptor;

    }

    // ----------------------------------------------------------------------
    // JBIPackage interface implementation
    // ----------------------------------------------------------------------

    public static boolean isDeploymentPackage(JBIDescriptor descriptor) {
        return descriptor.getServiceAssembly() != null;
    }

    public static boolean isInstallationPackage(JBIDescriptor descriptor) {
        return !isDeploymentPackage(descriptor);
    }

    /**
     * @see org.objectweb.petals.jbi.management.packaging.JBIPackage#getArchiveURI()
     */
    public URI getArchiveURI() {
        return packageLocation;
    }

    /**
     * @see org.objectweb.petals.jbi.management.packaging.JBIPackage#getDescriptor()
     */
    public JBIDescriptor getDescriptor() {

        return descriptor;
    }

    // ----------------------------------------------------------------------
    // implementation methods
    // ----------------------------------------------------------------------

    /**
     * @see org.objectweb.petals.jbi.management.packaging.JBIPackage#isDeploymentPackage()
     */
    public boolean isDeploymentPackage() {
        return isDeploymentPackage(descriptor);
    }

    /**
     * @see org.objectweb.petals.jbi.management.packaging.JBIPackage#isInstallationPackage()
     */
    public boolean isInstallationPackage() {
        return isInstallationPackage(descriptor);
    }

}
