/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: JoramAgentImpl.java,v 0.3 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.kernel.joram;

import java.io.File;
import java.io.IOException;
import java.net.ConnectException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import javax.naming.Binding;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;

import org.apache.commons.io.FileUtils;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.joram.client.jms.admin.AdminException;
import org.objectweb.joram.client.jms.admin.AdminModule;
import org.objectweb.petals.kernel.admin.ContainerInformation;
import org.objectweb.petals.kernel.admin.ContainerInformation.ContainerState;
import org.objectweb.petals.kernel.joram.util.JoramConfigurationBuilder;
import org.objectweb.petals.util.JNDIUtil;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.PropertyUtil;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

import static org.objectweb.petals.util.InterfaceNamesHelper.LOGGER_NAME;

import fr.dyade.aaa.agent.AgentServer;

/**
 * Joram Agent used to start and stop the Joram server
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * @author ddesjardins - eBMWebsourcing
 * @author Anass OUAZZANI - eBMWebsourcing
 */
public class JoramAgentImpl implements JoramAgent, LifeCycleController,
    BindingController {

    protected static final String CONTAINERS_REF = "containers";

    /**
     * Fractal client interfaces
     */
    private static final String[] CLIENT_INTERFACES = new String[] {};

    /**
     * Joram directory
     */
    private static final String JORAM_LOCATION = "joram";

    /**
     * Container UID
     */
    protected long containerUID;

    /**
     * Container Joram domain
     */
    protected String domain;

    /**
     * Container Joram domain port
     */
    protected String domainPort;

    /**
     * Monolog logger factory
     */
    protected LoggerFactory factory;

    /**
     * Component LifeCycle state.
     */
    protected String fcState;

    /**
     * Container host
     */
    protected String host;

    /**
     * Container id
     */
    protected String id;

    /**
     * Joram directory
     */
    protected File joramDir;

    /**
     * Logger Util
     */
    protected LoggingUtil log;

    /**
     * Monolog Logger
     */
    protected Logger logger;

    /**
     * Container name
     */
    protected String name;

    /**
     * Container joram password
     */
    protected String password;

    /**
     * Initial context
     */
    protected InitialContext rootContext;

    /**
     * Container Joram tcp port
     */
    protected String tcpPort;

    /**
     * Container Joram user name
     */
    protected String username;

    public JoramAgentImpl() {

    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
     *      java.lang.Object)
     */
    public void bindFc(String clientItfName, Object serverItf)
        throws NoSuchInterfaceException, IllegalBindingException,
        IllegalLifeCycleException {
        if (LOGGER_NAME.equals(clientItfName)) {
            logger = (Logger) serverItf;
            log = new LoggingUtil(logger);
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
     */
    public String getFcState() {
        return STARTED.equals(fcState) ? STARTED : STOPPED;
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() {
        return CLIENT_INTERFACES;
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String clientItfName)
        throws NoSuchInterfaceException {
        if (LOGGER_NAME.equals(clientItfName)) {
            return logger;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    /**
     * Retrieve the list of ContainerInformation by connecting to a JNDI
     * directory
     * 
     * @param jndiHost
     *            host of the JNDI directory
     * @param jndiPort
     *            port of the JNDI directory
     * @param jndiFactory
     *            factory of the JNDI directory
     * @param onlyStarted
     *            boolean to exclude stopped containers
     * @return List of ContainerInformation
     */
    public List<ContainerInformation> retrieveAllContainersInformation() {
        log.start();
        List<ContainerInformation> containers = new ArrayList<ContainerInformation>();
        try {
            InitialContext rootContext = new InitialContext(PropertyUtil
                .retrieveJNDIProperties());
            if (JNDIUtil.isBound(rootContext, "/", CONTAINERS_REF)) {
                NamingEnumeration<Binding> enums = rootContext
                    .listBindings(CONTAINERS_REF);
                while (enums.hasMoreElements()) {
                    Binding binding = (Binding) enums.next();
                    ContainerInformation containerInformation = (ContainerInformation) binding
                        .getObject();
                    if (containerInformation.isStarted()) {
                        containers.add(containerInformation);
                    }
                }
            }
        } catch (NamingException e) {
            log
                .debug("Problem while trying to retrieve all the containers information : context may not be initialized");
        }
        log.end();
        return containers;
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
     */
    public void startFc() throws IllegalLifeCycleException {
        log.call();
        startServer();
        fcState = STARTED;
    }

    /**
     * List of containers
     */
    protected List<ContainerInformation> containers = new ArrayList<ContainerInformation>();

    /**
     * Start the local Joram server
     */
    public void startServer() {
        
        boolean isInit = false;
        
        if (!isServerRunning()) {
        
            try {
                // retrieves the global network configuration if any
                containers = retrieveAllContainersInformation();
                
                // init the Joram server configuration from the
                // server.properties file
                initConfiguration();
                
                // Check if this server has already been started
                // and have a stored network configuration.
                boolean previsouslyStarted = isExistingConfiguration();

                if (!previsouslyStarted) {
                    log.debug("First start, create a a3servers.xml configuration file.");
                    // First start.
                    // We need to create an a3servers.xml file.

                    // if a global configuration exists (other servers are launched),
                    //   retreive this global network configuration
                    //   and create the a3servers.xml file with it
                    // otherwise, assume this server is the first,
                    //   create an a3servers.xml with this container only.
                    String configurationContent = null;
                    if (containers.size() > 0) {
                        /*
                         * DISTRIBUTED CONFIGURATION
                         */
                        log.debug("Retrieve the existing network configuration from one of the running server.");
                        configurationContent = retrieveDistributedConfiguration();
                    } else {
                        /*
                         * ALONE CONFIGURATION 
                         */
                        log.debug("Create an empty configuration as no server is runnning on the network.");
                        name = id;
                        SystemUtil.setContainerName(id);
                        // Get alone Joram configuration
                        configurationContent = JoramConfigurationBuilder
                            .buildFullMasterConf(domain, id, name, host,
                                username, password, domainPort, tcpPort, Long
                                    .toString(containerUID));
                    }

                    // Check if a former Joram server was not stop correctly
                    cleanJoramServerNotStoppedCorrectly();

                    // Write the configuration file with the
                    // distributed or alone configuration
                    FileUtils.writeStringToFile(new File(joramDir
                        + File.separator + "a3servers.xml"),
                        configurationContent, "UTF-8");

                    System.setProperty("fr.dyade.aaa.agent.A3CONF_DIR",
                        joramDir.getAbsolutePath());

                } else {
                    // Nothing to do, the Joram persistance repository 
                    // contains the last known configuration
                    log.debug("A stored network configuration as been found, restart with it.");
                }

                // Init the Joram server
                initJoramAgentServer();

                isInit = true;

                // Start the Joram server
                startJoramAgentServer();

                log.info("Joram server started at [host:port " + host + ":"
                    + domainPort + ",tcp port " + tcpPort + ", jndi port "
                    + ", domain name " + domain + ", server name " + name
                    + ", server id " + id + "\n");

            } catch (Exception e) {
                log.error("Problem while starting the Joram server", e);
                if (isInit) {
                    fr.dyade.aaa.agent.AgentServer.stop(true);
                }
            }
        }
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
     */
    public void stopFc() throws IllegalLifeCycleException {
        log.call();
        fcState = STOPPED;
        stopServer();
    }

    /**
     * Stop the local Joram server
     */
    public void stopServer() {
        log.start();
        updateContainerState(SystemUtil.getContainerName(),
            ContainerState.STOPPED);

        if (isServerRunning()) {
            fr.dyade.aaa.agent.AgentServer.stop();
        }
        log.end();
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
        IllegalBindingException, IllegalLifeCycleException {
        if (LOGGER_NAME.equals(clientItfName)) {
            logger = null;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }

    }

    /**
     * Add a Joram server to the Joram network
     * 
     * @param id
     *            id of the server to add
     * @param host
     *            host of the server to add
     * @param domain
     *            domain of the server to add
     * @param domainPort
     *            domain port of the server to add
     * @param tcpPort
     *            tcp port of the server to add
     * @throws ConnectException
     *             if a problems occurs while adding the server
     * @throws AdminException
     *             if a problems occurs while adding the server
     */
    protected void addJoramServer(int id, String host, String domain,
        int domainPort, String tcpPort) throws ConnectException, AdminException {
        AdminModule
            .addServer(
                id,
                host,
                domain,
                domainPort,
                name,
                new String[] {"org.objectweb.joram.mom.proxies.tcp.TcpProxyService"},
                new String[] {tcpPort});
    }

    /**
     * This method is used to clean the Joram directory
     * 
     * It will search for not correctly stopped Joram server
     */
    protected void cleanJoramServerNotStoppedCorrectly() {
        for (File file : joramDir.listFiles()) {
            if (file.isDirectory()) {
                // We are in a Joram backup directory
                boolean isLock = false;
                for (File joramFile : file.listFiles()) {
                    if ("lock".equals(joramFile.getName())) {
                        // We are looking for a lock file
                        isLock = true;
                        break;
                    }
                }
                if (isLock) {
                    // This Joram server was not shutdown correctly
                    // We need to remove the backup directory
                    try {
                        FileUtils.forceDelete(file);
                    } catch (IOException e) {
                        // Do nothing
                    }
                }
            }
        }
    }

    /**
     * Check if this server has been previously started and contains a
     * configuration (check that the JORAM persistance directory is not empty)
     * 
     * @return the server contains a previous network configuration
     */
    protected boolean isExistingConfiguration() {
        return (joramDir != null && joramDir.exists() && joramDir.listFiles().length > 0);
    }

    /**
     * Initialises the Joram configuration from the server.propertis file.
     * Creates the JORAM persistance directory if not exists
     * 
     * @throws IOException
     */
    protected void initConfiguration() {
        // Retreive the logger factory
        factory = Monolog
            .instanciateMonologFactory("org.objectweb.petals.util.monolog.wrapper.javalog.LoggerFactory");

        // Set the cache files directory for Joram
        joramDir = new File(SystemUtil.getPetalsInstallDirectory()
            + File.separator + JORAM_LOCATION + File.separator);
        if (!joramDir.exists()) {
            joramDir.mkdir();
        }

        // Retreive Joram properties
        domain = SystemUtil.getJoramDomain();
        id = SystemUtil.getJoramId();
        name = SystemUtil.getJoramId();
        host = SystemUtil.getHost();

        username = SystemUtil.getJoramUser();
        password = SystemUtil.getJoramPassword();
        tcpPort = SystemUtil.getJoramTCPPort();
        domainPort = SystemUtil.getJoramDomainPort();
        containerUID = SystemUtil.getContainerUID();
    }

    /**
     * @see AgentServer#init(short, String, LoggerFactory)
     */
    protected void initJoramAgentServer() throws Exception {
        log.call();
        fr.dyade.aaa.agent.AgentServer.init((short) Integer.parseInt(id),
            joramDir.getAbsolutePath() + File.separator + id, factory);
    }

    /**
     * Is the local joram server running or not
     * 
     * @return true if the server is running
     */
    protected boolean isServerRunning() {
        return 4 == fr.dyade.aaa.agent.AgentServer.getStatus();
    }

    /**
     * Is the specified server is registered in the Joram global configuration.
     * <b>AdminModule.connect required</b>
     * 
     * @return
     * @throws AdminException
     * @throws ConnectException
     */
    protected boolean isServerRegistered(String serverId)
        throws ConnectException, AdminException {
        boolean serverAlreadyExist = false;
        for (Iterator serverIdsIt = AdminModule.getServersIds().iterator(); serverIdsIt
            .hasNext()
            && !serverAlreadyExist;) {
            String aServerId = serverIdsIt.next().toString();
            serverAlreadyExist = (aServerId.equalsIgnoreCase(serverId));
        }
        return serverAlreadyExist;
    }

    /**
     * Retrieve a distribution configuration for this Joram server. <br>
     * We search for a valid Joram server on the network, <br>
     * then catch its configuration, <br>
     * add our container into it if necessary (if it has been previously <br>
     * started, it is already in this conf), <br>
     * and return this new configuration (as an XML string)
     * 
     * @return String containing the Joram configuration
     */
    protected String retrieveDistributedConfiguration() {
        String conf = null;

        if (containers.size() > 0) {
            // Loop on the containers to connect to the AdminModule
            Iterator<ContainerInformation> containersIt = containers.iterator();
            boolean fail = true;
            while (fail && containersIt.hasNext()) {
                try {
                    ContainerInformation information = containersIt.next();
                    log.debug("Trying to connect to Joram server on "
                        + information.getHost() + ":"
                        + information.getJoramTCPPort());
                    AdminModule.connect(information.getHost(), Integer
                        .parseInt(information.getJoramTCPPort()), information
                        .getJoramLogin(), information.getJoramPassword(), 3);

                    // Recover the configuration
                    conf = AdminModule.getConfiguration();

                    // if our server has not been already recorded in the global
                    // conf, add it
                    if (!isServerRegistered(id)) {
                        addJoramServer(Integer.parseInt(id), host, domain,
                            Integer.parseInt(domainPort), tcpPort);
                    }
                    AdminModule.disconnect();

                    // Exit the loop if everything is OK
                    fail = false;
                } catch (Exception e) {
                    log.warning("Impossible to connect to Joram, try another",
                        e);
                }
            }
            if (conf != null) {
                // Set correct parameters in the Conf if necessary
                conf = JoramConfigurationBuilder.buildFullConfiguration(conf,
                    domain, id, name, host, username, password, domainPort,
                    tcpPort, Long.toString(containerUID));
            } else {
                throw new RuntimeException(
                    "Unable to connect to other Petals container, so stop");
            }
        } else {
            conf = JoramConfigurationBuilder.buildFullMasterConf(domain, id,
                name, host, username, password, domainPort, tcpPort, Long
                    .toString(containerUID));
        }
        return conf;
    }

    /**
     * Retrieve an initial context
     * 
     * @param outProperties
     *            properties of the JNDI directory
     * @return Initial context
     * @throws NamingException
     *             if a problem occurs
     */
    protected InitialContext retrieveInitialContext(Properties outProperties)
        throws NamingException {
        return new InitialContext(outProperties);
    }

    /**
     * @see AgentServer#start()
     */
    protected void startJoramAgentServer() throws Exception {
        log.call();
        fr.dyade.aaa.agent.AgentServer.start();
    }

    /**
     * Update the state of a container
     * 
     * @param containerName
     *            name of the container to update
     * @param state
     *            state of the container
     */
    protected void updateContainerState(String containerName,
        ContainerState state) {
        log.start();
        if (containerName == null) {
            throw new IllegalArgumentException(
                "Container name must be non null");
        }
        if (!(!state.equals(ContainerState.STARTED) || !state
            .equals(ContainerState.STOPPED))) {
            throw new IllegalArgumentException("State must be "
                + ContainerState.STARTED + " or " + ContainerState.STOPPED);
        }
        try {
            InitialContext rootContext = new InitialContext(PropertyUtil
                .retrieveJNDIProperties());
            if (JNDIUtil.isBound(rootContext, "/", CONTAINERS_REF)) {
                if (JNDIUtil
                    .isBound(rootContext, CONTAINERS_REF, containerName)) {
                    Context containersContext = (Context) rootContext
                        .lookup(CONTAINERS_REF);
                    ContainerInformation containerInformation = (ContainerInformation) containersContext
                        .lookup(containerName);
                    containerInformation.setState(state);
                    containersContext.rebind(containerName,
                        containerInformation);
                }
            }
        } catch (Exception e) {
            log.error("Problem while updating container state", e);
        }
        log.end();
    }

    /**
     * Shutdown the Joram server. It is unreferenced the distributed joram
     * server configuration, and its <code>ContainerInformation</code> is
     * removed from the distributed jndi server.
     */
    public void shutdown() throws Exception {

        // algo : connect to this server,
        // remove ourselves from the configuration,
        // this remove is automaticcaly propagated
        // to other Servers

        try {
            AdminModule.connect(host, Integer.parseInt(tcpPort), username,
                password, 3);
            AdminModule.removeServer(Integer.parseInt(SystemUtil.getJoramId()));
            AdminModule.disconnect();

            removeFromContainersInformation();
        } catch (Exception e) {
            log.error("Problem while connecting to this Joram server", e);
        }

        // algo: tell each server that i am removed
        /*
         * String conf = null; try { AdminModule.connect(host,
         * Integer.parseInt(tcpPort), username, password, 3); conf =
         * AdminModule.getConfiguration(); AdminModule.disconnect(); } catch
         * (Exception e) { log.error("Problem while connecting to this Joram
         * server", e); }
         * 
         * List<JoramServer> joramList = JoramConfigurationBuilder
         * .retrieveJoramServers(conf, SystemUtil.getJoramId());
         * 
         * Iterator<JoramServer> iterator = joramList.iterator(); boolean notOK =
         * true; while (notOK && iterator.hasNext()) { try { JoramServer server =
         * iterator.next(); AdminModule.connect(server.getHost(),
         * Integer.parseInt(server .getPort()), username, password, 3);
         * AdminModule.removeServer(Integer.parseInt(SystemUtil .getJoramId()));
         * AdminModule.disconnect(); notOK = false; } catch (Exception e) {
         * e.printStackTrace(); } }
         */

    }

    /**
     * Remove this container from JNDI
     */
    protected void removeFromContainersInformation() {
        log.start();
        String containerName = SystemUtil.getContainerName();
        try {
            InitialContext rootContext = new InitialContext(PropertyUtil
                .retrieveJNDIProperties());
            if (JNDIUtil.isBound(rootContext, "/", CONTAINERS_REF)) {
                if (JNDIUtil
                    .isBound(rootContext, CONTAINERS_REF, containerName)) {
                    Context containersContext = (Context) rootContext
                        .lookup(CONTAINERS_REF);
                    containersContext.unbind(containerName);
                }
            }
        } catch (Exception e) {
            log.error("Problem while removing container information", e);
        }
    }

}
