/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Launcher.java 461 2006-05-24 12:12:45Z ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.system.launch;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import javax.jbi.management.DeploymentServiceMBean;
import javax.jbi.management.InstallationServiceMBean;
import javax.management.InstanceAlreadyExistsException;
import javax.management.InstanceNotFoundException;
import javax.management.JMException;
import javax.management.MBeanException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.NotCompliantMBeanException;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.management.modelmbean.InvalidTargetObjectTypeException;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.adl.FactoryFactory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.jmx.agent.AdminAttributes;
import org.objectweb.fractal.jmx.agent.Introspector;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.util.monolog.Monolog;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.management.service.AdminServiceMBean;
import org.objectweb.petals.jbi.management.service.ManagementException;
import org.objectweb.petals.jbi.management.systemstate.SystemState;
import org.objectweb.petals.jbi.transport.TransportException;
import org.objectweb.petals.jbi.transport.Transporter;
import org.objectweb.petals.kernel.admin.service.PetalsAdminServiceMBean;
import org.objectweb.petals.system.terminate.ShutdownHook;
import org.objectweb.petals.util.JNDIUtil;
import org.objectweb.petals.util.SystemUtil;

/**
 * This class is used to start the Petals platform
 * 
 * @author alouis, ddesjardins, Rafael Marins
 * @version $Rev: 461 $ $Date: 2006-05-24 14:12:45 +0200 (mer., 24 mai 2006) $
 */
public class Launcher {

    /**
     * Petals composite
     */
    protected Component petalsComposite;

    /**
     * 
     */
    public Launcher() {
        super();
    }

    /**
     * @param args
     */
    public static void main(String[] args) {
        try {
            if (args.length > 0 && "stop".equals(args[0])) {
                Launcher launcher = new Launcher();
                launcher.stop(args);
            } else if (args.length > 0 && "JNDIBrowser".equals(args[0])) {
                System.out.println(JNDIUtil.browseJNDI(args));
            } else {
                Launcher launcher = new Launcher();
                launcher.run(args);
            }
        } catch (Throwable e) {
            e.printStackTrace(System.err);
            Component kernelComp = new Launcher().getPetalsComposite();
            if (kernelComp != null) {
                try {
                    LifeCycleController lcc = Fractal
                        .getLifeCycleController(kernelComp);
                    if (lcc.getFcState().equals(LifeCycleController.STARTED)) {
                        lcc.stopFc();
                        System.out.println("Petals is " + lcc.getFcState());
                    }
                } catch (IllegalLifeCycleException e1) {
                    System.err.println("Problem while stopping Petals");
                } catch (NoSuchInterfaceException e2) {
                    System.err.println("Problem while stopping Petals");
                }
            }
            System.err.println("Petals is stopped !");
            System.exit(-1);
        }
    }

    public Component getPetalsComposite() {
        return petalsComposite;
    }

    
    /**
     * A utility function allowing to get a component from a content controller
     * 
     * @param cc
     *            ContentController
     * @param name
     *            component name
     * @return
     */
    protected Component getComponentByName(ContentController contentController,
        String name) {
        // The component to be returned
        Component c = null;
        // List content controller subcomponents
        Component[] comps = contentController.getFcSubComponents();
        for (int i = 0; i < comps.length; i++) {
            Component component = comps[i];
            try {
                String componentName = Fractal.getNameController(component)
                    .getFcName();
                if (componentName.equals(name)) {
                    c = component;
                    break;
                }
            } catch (NoSuchInterfaceException e) {
                // Do nothing
            }
        }
        return c;
    }

    /**
     * Initialize Fractal
     * 
     * @throws Exception
     */
    protected void initializeFractal() throws Exception {
        // check required fractal properties
        System.setProperty("fractal.provider",
            "org.objectweb.fractal.julia.Julia");
        System.setProperty("julia.loader",
            "org.objectweb.fractal.julia.loader.DynamicLoader");
        System.setProperty("julia.config", "julia.cfg");

        Map<String, Object> context = new HashMap<String, Object>();
        // Create Fractal ADL factories
        Factory factory = FactoryFactory.getFactory(
            FactoryFactory.FRACTAL_BACKEND, context);
        // System.setSecurityManager(new SecurityManager());
        // Registry.createRegistry(Registry.DEFAULT_PORT, context);
        // Create composite from ADL
        String compositeName = "Petals";
        petalsComposite = (Component) factory.newComponent(compositeName,
            context);

    }

    /**
     * 
     * @throws IOException
     */
    protected void initializeMonolog() throws IOException {
        // Configure Monolog
        URL url1 = this.getClass().getResource("/loggers.properties");
        Properties p1 = new Properties();
        p1.load(url1.openStream());
        String petalsLF = p1.getProperty("handler.petalsLF.output");
        if (petalsLF != null) {
            p1.setProperty("handler.petalsLF.output", new File(SystemUtil
                .getPetalsInstallDirectory()
                + File.separator + "logs" + File.separator + petalsLF)
                .getAbsolutePath());
        }
        String joramLF = p1.getProperty("handler.joramLF.output");
        if (joramLF != null) {
            p1.setProperty("handler.joramLF.output", new File(SystemUtil
                .getPetalsInstallDirectory()
                + File.separator + "logs" + File.separator + joramLF)
                .getAbsolutePath());
        }
        Monolog.initialize();
        Monolog.loadMonologConfiguration(p1);
    }

    /**
     * @see MBeanServerConnection#invoke(ObjectName, String, Object[], String[])
     */
    protected void invokeStopFc(MBeanServerConnection connection,
        ObjectName result) throws InstanceNotFoundException, MBeanException,
        ReflectionException, IOException {
        connection.invoke(result, "stopFc", new Object[0], new String[0]);
    }

    

   
    /**
     * @see JMXConnectorFactory#connect(JMXServiceURL, Map)
     */
    protected JMXConnector retrieveJMXConnector(JMXServiceURL url,
        Map<String, ?> args) throws IOException {
        return JMXConnectorFactory.connect(url, args);
    }

    protected void startPetals() throws NoSuchInterfaceException,
        IllegalLifeCycleException {
        String componentName = "jndi-server";
        ContentController contentController = Fractal
            .getContentController(petalsComposite);
        Component jndiServer = getComponentByName(contentController,
            componentName);

        LifeCycleController jndiServerLifeCycleController = Fractal
            .getLifeCycleController(jndiServer);
        jndiServerLifeCycleController.startFc();

        // Start the composite
        LifeCycleController lcc = Fractal
            .getLifeCycleController(petalsComposite);
        lcc.startFc();
    }

    protected void run(String[] args) throws Exception {

        // set the compatible SUID for the Qname
        System.setProperty(
            "com.sun.xml.namespace.QName.useCompatibleSerialVersionUID", "1.0");

        try {
            initializeMonolog();
        } catch (IOException e) {
            throw new PetalsException(
                "Monolog can't be correctly initialized !");
        }

        Monolog.initialize().getLogger("root0.Petals.launcher");

        System.setProperty("javax.management.builder.initial",
            "mx4j.server.MX4JMBeanServerBuilder");

        // Initialize Fractal
        initializeFractal();

        // Start Petals
        startPetals();

        // expose JBI MBean objectin the JMX Server
        registerMBeans();

        
        ContentController contentController = Fractal
        .getContentController(petalsComposite);
        
        // recover all entities
        recoverSystem(contentController);

        // tell the Transporter to start Listening
        startListening(contentController);

        // Set up the shutdown hook
        LifeCycleController lcc = Fractal
        .getLifeCycleController(petalsComposite);
    
        Runtime.getRuntime().addShutdownHook(
            new ShutdownHook(lcc, contentController));

        System.out.println("##### Petals container correctly started #####");

        // show the commandLine mode if asked
        if (args != null && args.length > 0
            && "-console".equalsIgnoreCase(args[0])) {
            commandLineMode(contentController);
        }
    }

    /**
     * Stop Petals container (to use when you launch Petals from Eclipse)
     * 
     * @param arguments
     */
    protected void stop(String[] arguments) {
        try {
            String jmxHost = SystemUtil.getHost();
            String jmxPort = SystemUtil.getJmxPort();
            String username = SystemUtil.getJmxUser();
            String password = SystemUtil.getJmxPassword();

            JMXConnector connector = null;
            Properties env = System.getProperties();
            env.put("java.naming.factory.initial",
                "com.sun.jndi.rmi.registry.RegistryContextFactory");
            int i = 0;
            for (; i < arguments.length; i++) {
                if ("-host".equals(arguments[i])) {
                    i++;
                    jmxHost = arguments[i];
                } else if ("-port".equals(arguments[i])) {
                    i++;
                    jmxPort = arguments[i];
                }
            }

            Map<String, Object> args = new HashMap<String, Object>();
            if (!"".equals(username)) {
                if (password == null) {
                    password = "";
                }
                String[] credentials = new String[] {username, password};
                args.put("jmx.remote.credentials", credentials);
            }
            String urlStr = "service:jmx:rmi:///jndi/rmi://" + jmxHost + ":"
                + jmxPort + "/management/rmi-jmx-connector";
            JMXServiceURL url = new JMXServiceURL(urlStr);
            connector = retrieveJMXConnector(url, args);
            MBeanServerConnection connection = connector
                .getMBeanServerConnection();

            ObjectName result = null;
            String pattern = "FC/Petals@*:itf=lifecycle-controller";
            Set objNames;
            ObjectName objName = new ObjectName(pattern);
            objNames = connection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                result = (ObjectName) objNames.iterator().next();
            }
            if (result != null) {
                invokeStopFc(connection, result);
            }
        } catch (Exception e) {
            System.err.println("The container Petals must be disconnected");
        }
    }

    protected void registerMBeans() throws NoSuchInterfaceException, InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException, MalformedObjectNameException, NullPointerException, JMException, InvalidTargetObjectTypeException{

        String componentName = "agent-jmx";
        ContentController contentController = Fractal
        .getContentController(petalsComposite);
        Component agent = getComponentByName(contentController, componentName);
        
        String interfaceName = "admin";
        Introspector.CURRENCY_TIME_LIMIT = "-1";
        interfaceName = "attribute-controller";
        AdminAttributes adminAttributes = (AdminAttributes) agent
            .getFcInterface(interfaceName);
        
        MBeanServer jmxServer = adminAttributes.getRawMBeanServer();
    
        // register the fractal lifecycle controller
        LifeCycleController lcc = Fractal
        .getLifeCycleController(petalsComposite);
    
        jmxServer.registerMBean(
            Introspector.createMBean(lcc),
            new ObjectName("FC/Petals@" + lcc.hashCode()
                + ":itf=lifecycle-controller"));
        
        // register JBI AdminMBean
        Component jbi = getComponentByName(contentController, "jbi");
        ContentController contentControllerForJBI = Fractal
            .getContentController(jbi);
        Component systemState = getComponentByName(contentControllerForJBI,
            "admin-impl");
        Object admin = systemState.getFcInterface("service");
        
        jmxServer.registerMBean(Introspector.createMBean(admin,AdminServiceMBean.class),
            new ObjectName("Petals:name=Admin,type=service"));

        // register JBI DeployMBean
        systemState = getComponentByName(contentControllerForJBI,
            "deployment-impl");
        Object deployment = systemState.getFcInterface("service");
        
        jmxServer.registerMBean(Introspector.createMBean(deployment,DeploymentServiceMBean.class),
            new ObjectName("Petals:name=Deployment,type=service"));

        // register JBI InstallMBean
        systemState = getComponentByName(contentControllerForJBI,
            "installation-impl");
        Object install = systemState.getFcInterface("service");
        
        jmxServer.registerMBean(Introspector.createMBean(install,InstallationServiceMBean.class) ,
            new ObjectName("Petals:name=Installation,type=service"));

        // register Petals Admin
        systemState = getComponentByName(contentController,
            "petalsadmin-service");
        Object petalsService = systemState.getFcInterface("service");
        
        jmxServer.registerMBean(Introspector.createMBean(petalsService,PetalsAdminServiceMBean.class) ,
            new ObjectName("Petals:name=PetalsAdmin,type=service"));


        
    }
    /**
     * Recover the system state. All state elements are recovered (components,
     * share libraries, service assemblies)
     * 
     * @param contentContoller
     * @throws NoSuchInterfaceException
     * @throws ManagementException
     */
    protected void recoverSystem(ContentController contentContoller)
        throws NoSuchInterfaceException, ManagementException {
        Component jbi = getComponentByName(contentContoller, "jbi");
        ContentController contentControllerForJBI = Fractal
            .getContentController(jbi);
        Component systemState = getComponentByName(contentControllerForJBI,
            "systemstate-impl");
        SystemState systemStateService = (SystemState) systemState
            .getFcInterface("service");
        systemStateService.recoverAllEntities();
    }

    /**
     * Start listening for incoming messages. This is only activated after all
     * the components has been restarted.
     * 
     * @param contentContoller
     * @throws NoSuchInterfaceException
     * @throws ManagementException
     * @throws TransportException
     */
    protected void startListening(ContentController contentContoller)
        throws NoSuchInterfaceException, ManagementException,
        TransportException {
        Component jbi = getComponentByName(contentContoller, "jbi");
        ContentController contentControllerForJBI = Fractal
            .getContentController(jbi);
        Component systemState = getComponentByName(contentControllerForJBI,
            "transporter-impl");
        Transporter transporter = (Transporter) systemState
            .getFcInterface("service");
        transporter.startListening();
    }
    
    /**
     * Show a command line to interact with Petals
     * TODO we pass the PetalsAdmin object to the CommandReader
     * to perform some actions. Find a better way.
     * @param contentController used to retreive PetalsAdmin
     * @throws NoSuchInterfaceException 
     */
    protected void commandLineMode(ContentController contentController) throws NoSuchInterfaceException{
        Component systemState = getComponentByName(contentController,"petalsadmin-service");
        PetalsAdminServiceMBean petalsService = (PetalsAdminServiceMBean)systemState.getFcInterface("service");
  
        CommandReader console = new CommandReader(petalsService);
        
        console.read();
    }

}
