/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: AutoLoaderImplTest.java 3:54:28 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.autoload;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import junit.framework.TestCase;

import org.apache.commons.io.FileUtils;
import org.easymock.classextension.EasyMock;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.management.service.DeploymentServiceImpl;
import org.objectweb.petals.jbi.management.service.DeploymentService;
import org.objectweb.petals.jbi.management.service.InstallationServiceImpl;
import org.objectweb.petals.jbi.management.service.InstallationService;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerImpl;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerService;
import org.objectweb.petals.jbi.management.service.PackageHandler;
import org.objectweb.petals.jbi.management.systemstate.ComponentState;
import org.objectweb.petals.jbi.management.systemstate.ServiceAssemblyState;
import org.objectweb.petals.jbi.management.systemstate.SharedLibraryState;
import org.objectweb.petals.jbi.management.systemstate.SystemState;
import org.objectweb.petals.jbi.management.systemstate.SystemStateImpl;
import org.objectweb.petals.repository.RepositoryImpl;
import org.objectweb.petals.repository.RepositoryService;
import org.objectweb.petals.tools.jbicommon.descriptor.ComponentDescription;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.util.InterfaceNamesHelper;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.wrapper.printwriter.LoggerImpl;

import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;
import static org.easymock.EasyMock.expect;
/**
 * Test of the AutoLoaderImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 * @author wjoseph - eBMWebsourcing
 */
public class AutoLoaderImplTest extends TestCase {

    private AutoLoaderImpl autoLoaderImpl;

    private String baseDir;

    public void setUp() {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        System.setProperty("petals.home", new File(baseDir + "target")
            .getAbsolutePath());
        new File(baseDir + "target" + File.separator + "lib").mkdir();
        new File(baseDir + "target" + File.separator + "lib" + File.separator
            + "petals-1.0-SNAPSHOT.jar").mkdir();
        autoLoaderImpl = new AutoLoaderImpl();
    }

    public void testBindFc() throws NoSuchInterfaceException,
        IllegalBindingException, IllegalLifeCycleException {
        InstallationServiceImpl installationService = new InstallationServiceImpl();
        autoLoaderImpl.bindFc(InterfaceNamesHelper.JBI_INSTALLATION_NAME,
            installationService);
        assertEquals(autoLoaderImpl
            .lookupFc(InterfaceNamesHelper.JBI_INSTALLATION_NAME),
            installationService);

        Logger logger = Monolog.initialize().getLogger("test");
        autoLoaderImpl.bindFc(InterfaceNamesHelper.LOGGER_NAME, logger);
        assertEquals(autoLoaderImpl.lookupFc(InterfaceNamesHelper.LOGGER_NAME),
            logger);

        DeploymentServiceImpl deploymentService = new DeploymentServiceImpl();
        autoLoaderImpl.bindFc(InterfaceNamesHelper.JBI_DEPLOYMENT_NAME,
            deploymentService);
        assertEquals(autoLoaderImpl
            .lookupFc(InterfaceNamesHelper.JBI_DEPLOYMENT_NAME),
            deploymentService);

        LifeCycleManagerImpl lifeCycleManagerImpl = new LifeCycleManagerImpl();
        autoLoaderImpl.bindFc(InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME,
            lifeCycleManagerImpl);
        assertEquals(autoLoaderImpl
            .lookupFc(InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME),
            lifeCycleManagerImpl);

        RepositoryImpl repositoryImpl = new RepositoryImpl();
        autoLoaderImpl.bindFc(InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME,
            repositoryImpl);
        assertEquals(autoLoaderImpl
            .lookupFc(InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME),
            repositoryImpl);

        SystemStateImpl systemStateImpl = new SystemStateImpl();
        autoLoaderImpl.bindFc(InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME,
            systemStateImpl);
        assertEquals(autoLoaderImpl
            .lookupFc(InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME),
            systemStateImpl);

        try {
            autoLoaderImpl.bindFc("test", null);
            fail();
        } catch (Exception e) {
            // Do nothing
        }

        try {
            autoLoaderImpl.lookupFc("test");
            fail();
        } catch (Exception e) {
            // Do nothing
        }
    }

    public void testGetFcState() {
        assertEquals(autoLoaderImpl.getFcState(), LifeCycleController.STOPPED);
    }

    public void testInit() throws IOException {
        autoLoaderImpl.logger = Monolog.initialize().getLogger("test");
        autoLoaderImpl.init();
        assertEquals(AutoLoaderImpl.getInstallDirectory(), new File(baseDir
            + "target" + File.separator + "install"));
        assertEquals(AutoLoaderImpl.getInstalledDirectory(), new File(baseDir
            + "target" + File.separator + "installed"));
        assertEquals(AutoLoaderImpl.getWorkDirectory(), new File(baseDir
            + "target" + File.separator + "work"));
        assertEquals(AutoLoaderImpl.getUninstalledDirectory(), new File(baseDir
            + "target" + File.separator + "uninstalled"));
    }

    public void testGetInstallDirectory () {
        /*
         * Test Case  1
         */
        File install = new File("");
        AutoLoaderImpl.installDirectory = install;  
        /*
         * Run Test Case  1
         */
        assertEquals("Wrong invokation result",AutoLoaderImpl.getInstallDirectory(),install);
    }
 
    public void testGetInstalledDirectory () {
        /*
         * Test Case  1
         */
        File installed = new File("");
        AutoLoaderImpl.installedDirectory = installed;  
        /*
         * Run Test Case  1
         */
        assertEquals("Wrong invokation result",AutoLoaderImpl.getInstallDirectory(),installed);
    }
    
    public void testGetUninstalDirectory () {
        /*
         * Test Case  1
         */
        File uninstalled = new File("");
        AutoLoaderImpl.uninstalledDirectory = uninstalled;  
        /*
         * Run Test Case  1
         */
        assertEquals("Wrong invokation result",AutoLoaderImpl.getUninstalledDirectory(),uninstalled);
    }
    
    public void testGetWorkDirectory () {
        /*
         * Test Case  1
         */
        File work = new File("");
        AutoLoaderImpl.workDirectory = work;  
        /*
         * Run Test Case  1
         */
        assertEquals("Wrong invokation result",AutoLoaderImpl.getWorkDirectory(),work);
    }
    
    public void tearDown() {
        System.clearProperty("petals.home");
    }

    public void testListFc() {
        /*
         * Test Case 1
         */
        /*
         * Run Test Case 1
         */
        assertEquals("Wrong invokation result",AutoLoaderImpl.CLIENT_INTERFACES, autoLoaderImpl.listFc());
    }
    
    public void testStart() throws IOException {
        testInit();
        autoLoaderImpl.log = new LoggingUtil(autoLoaderImpl.logger);
        autoLoaderImpl.start();
        assertNotNull(autoLoaderImpl.installDirectoryScanTimer);
        assertNotNull(autoLoaderImpl.installedDirectoryScanTimer);
    }
    
    public void testStartFc() {
        /*
         * Test Case 1
         * No exception is raised 
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        Method method2 = null;
        try {
            method1 = AutoLoaderImpl.class.getDeclaredMethod("init", new Class[]{});
            method2 = AutoLoaderImpl.class.getDeclaredMethod("start", new Class[]{});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        autoLoaderImpl = createMock(AutoLoaderImpl.class,new Method[]{method1, method2});
        /*
         * Initialise Mocks
         */
        try {
            autoLoaderImpl.init();
            autoLoaderImpl.start();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialise Mocks");
        }
        replay(autoLoaderImpl);
        /*
         * Run Test Case 1
         */
        try {
        autoLoaderImpl.startFc();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        verify(autoLoaderImpl);
        assertEquals(autoLoaderImpl.getFcState(), AutoLoaderImpl.STARTED);
        
        /*
         * Test Case 2
         * An exception is raised during init 
         */
        autoLoaderImpl = new AutoLoaderImpl() {
            @Override
            public void init() throws IOException {
                throw new IOException("init testing exception");
            }
        };
        LoggingUtil log = createMock(LoggingUtil.class);
        autoLoaderImpl.log = log;
        /*
         * Run Test Case 2
         */
        try {
        autoLoaderImpl.startFc();
        fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wring exception message",e.getMessage().contains("init testing exception"));
        }
    }

    public void testStopFc() {
        /*
         * Test Case 1
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        try {
            method1 = AutoLoaderImpl.class.getDeclaredMethod("stop", new Class[]{});
        } catch (Exception e) {
            e.printStackTrace();
            fail ("Could not retrieve mocked methods");
        }
        autoLoaderImpl = createMock(AutoLoaderImpl.class, new Method[]{method1});
        /*
         * Initialise Mocks
         */
        autoLoaderImpl.stop();
        replay(autoLoaderImpl);
        /*
         * Run Test Case 1
         */
        try {
        autoLoaderImpl.stopFc();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        verify(autoLoaderImpl);
        assertEquals("The autoloderImpl state was not changed",autoLoaderImpl.getFcState(), AutoLoaderImpl.STOPPED);
    }
    
    public void testInstallComponent() throws IOException, PetalsException,
        MalformedObjectNameException, NullPointerException,
        InstanceNotFoundException, MBeanException, ReflectionException {
        FileUtils.copyFileToDirectory(new File(baseDir + "src" + File.separator
            + "test-data" + File.separator + "autoload" + File.separator
            + "toInstall.zip"), new File(baseDir + "target"));
        testInit();

        JBIDescriptor descriptor = EasyMock.createMock(JBIDescriptor.class);
        ComponentDescription componentDescription = EasyMock
            .createMock(ComponentDescription.class);
        EasyMock.replay(componentDescription);

        EasyMock.expect(descriptor.getComponent()).andReturn(
            componentDescription).anyTimes();
        EasyMock.replay(descriptor);

        PackageHandler packageHandler = EasyMock
            .createMock(PackageHandler.class);
        EasyMock.expect(
            packageHandler.loadDescriptor(new File(baseDir + "target"
                + File.separator + "work" + File.separator + "toInstall.zip")
                .toURI())).andReturn(descriptor).anyTimes();
        EasyMock.replay(packageHandler);

        LifeCycleManagerImpl lifeCycleManagerImpl = EasyMock
            .createMock(LifeCycleManagerImpl.class);

        MockMbeanServer server = new MockMbeanServer();

        EasyMock.expect(lifeCycleManagerImpl.getMBeanServer())
            .andReturn(server).anyTimes();

        EasyMock.replay(lifeCycleManagerImpl);

        InstallationServiceImpl installationService = EasyMock
            .createMock(InstallationServiceImpl.class);
        EasyMock.expect(
            installationService.loadNewInstaller(new File(baseDir + "target"
                + File.separator + "work" + File.separator + "toInstall.zip")
                .toURI().toString())).andReturn(
            new ObjectName("test@test:name=installer")).anyTimes();
        EasyMock.replay(installationService);

        autoLoaderImpl.installationService = installationService;
        autoLoaderImpl.managerService = lifeCycleManagerImpl;
        autoLoaderImpl.packageHandler = packageHandler;
        autoLoaderImpl.log = new LoggingUtil(autoLoaderImpl.logger);
        autoLoaderImpl.install(new File[] {new File(baseDir + "target"
            + File.separator + "toInstall.zip")});

        assertTrue(server.isInvokeStart());
    }
    
    public void testUnbindFc() {
        /*
         * Test Case 1 The parameter clientItfName is null, an exception will be
         * raised
         */
        /*
         * Create Test parameters
         */
        String clientItfName = null;
        Object serverItf = null;
        /*
         * Run Test Case 1
         */
        try {
            autoLoaderImpl.unbindFc(clientItfName);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The parameter clientItfName match the logger interface
         * name The logger attribute of autoLoaderImpl will be set to null
         */
        serverItf = createMock(LoggerImpl.class);
        autoLoaderImpl.logger = (Logger) serverItf;
        clientItfName = InterfaceNamesHelper.LOGGER_NAME;
        /*
         * Run Test Case 2
         */
        try {
            autoLoaderImpl.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("logger attribute was not properly set",
                autoLoaderImpl.logger, null);

        /*
         * Test Case 3 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME The managerService
         * attribute of autoLoaderImpl will be set to null
         */
        serverItf = createMock(LifeCycleManagerImpl.class);
        autoLoaderImpl.managerService = (LifeCycleManagerService) serverItf;
        clientItfName = InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME;
        /*
         * Run Test Case 3
         */
        try {
            autoLoaderImpl.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                autoLoaderImpl.managerService, null);

        /*
         * Test Case 4 The parameter clientItfName match the
         * InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME The repositorySrv
         * attribute of autoLoaderImpl will be set to null
         */
        serverItf = createMock(RepositoryImpl.class);
        clientItfName = InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME;
        autoLoaderImpl.repositorySrv = (RepositoryService) serverItf;
        /*
         * Run Test Case 4
         */
        try {
            autoLoaderImpl.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                autoLoaderImpl.repositorySrv, null);

        /*
         * Test Case 5 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_ADMIN_NAMEThe adminSrv attribute of
         * autoLoaderImpl will be set to null
         */
        serverItf = createMock(InstallationServiceImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_INSTALLATION_NAME;
        autoLoaderImpl.installationService =  (InstallationService) serverItf;
        /*
         * Run Test Case 5
         */
        try {
            autoLoaderImpl.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("installationService attribute was not properly set",
                autoLoaderImpl.installationService, null);

        /*
         * Test Case 6 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME The recoverySrv attribute
         * of autoLoaderImpl will be set to null
         */
        serverItf = createMock(SystemStateImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME;
        autoLoaderImpl.recoverySrv = (SystemState) serverItf;
        /*
         * Run Test Case 6
         */
        try {
            autoLoaderImpl.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                autoLoaderImpl.recoverySrv, null);

        /*
         * Test Case 7 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME The recoverySrv
         * attribute of autoloaderImpl will be set to serverItf value
         */
        serverItf = createMock(DeploymentService.class);
        clientItfName = InterfaceNamesHelper.JBI_DEPLOYMENT_NAME;
        autoLoaderImpl.deploymentService =  (DeploymentService) serverItf;
        /*
         * Run Test Case 7
         */
        try {
            autoLoaderImpl.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("deploymentService attribute was not properly set",
                autoLoaderImpl.deploymentService, null);
    }
    
    public void testUninstall() {
        /*
         * Test Case 1
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        try {
            method1 = AutoLoaderImpl.class.getDeclaredMethod("performCompUninstall", new Class[]{String.class});
            method2 = AutoLoaderImpl.class.getDeclaredMethod("performSAUninstall", new Class[]{String.class});
            method3 = AutoLoaderImpl.class.getDeclaredMethod("performSLUninstall", new Class[]{String.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        autoLoaderImpl = createMock(AutoLoaderImpl.class,new Method[]{method1, method2, method3});
        /*
         * Create Test parameters
         */
        List<ComponentState> compStates = new ArrayList<ComponentState>();
        ComponentState compState = createMock(ComponentState.class);
        compStates.add(compState);
        List<SharedLibraryState> slStates = new ArrayList<SharedLibraryState>();
        SharedLibraryState slState = createMock(SharedLibraryState.class);
        slStates.add(slState);
        List<ServiceAssemblyState> saStates = new ArrayList<ServiceAssemblyState>();
        ServiceAssemblyState saState = createMock(ServiceAssemblyState.class);
        saStates.add(saState);
        String compName = "component Name";
        String slName = " shared library name";
        String saName = "service assembly name";
        String url = "myUrl";
        File file = new File("");
        URI uri = file.toURI();
        file = new File(uri);
        PackageHandler packageHandler = createMock(PackageHandler.class);
        List<File> fileToUninstall = new ArrayList<File>();
        fileToUninstall.add(file);
        LoggingUtil log = createMock(LoggingUtil.class);
        SystemState systemState = createMock(SystemState.class);
        /*
         * Initialize Mocks
         */
        expect(compState.getName()).andReturn(compName);
        expect(compState.getArchiveURL()).andReturn(url);
        expect(slState.getName()).andReturn(slName);
        expect(slState.getArchiveURL()).andReturn(url);
        expect(saState.getName()).andReturn(saName);
        expect(saState.getArchiveURL()).andReturn(url);
        expect(packageHandler.processAndGetPackageURI(url, false)).andReturn(uri).anyTimes();
        autoLoaderImpl.performCompUninstall(compName);
        autoLoaderImpl.performSAUninstall(saName);
        autoLoaderImpl.performSLUninstall(slName);
        expect(systemState.retrieveAllComponentStates()).andReturn(compStates);
        expect(systemState.retrieveAllSharedLibraryStates()).andReturn(slStates);
        expect(systemState.retrieveAllServiceAssemblyStates()).andReturn(saStates);
        replay(compState);
        replay(slState);
        replay(saState);
        replay(packageHandler);
        replay(autoLoaderImpl);
        replay(systemState);
        autoLoaderImpl.packageHandler = packageHandler;
        autoLoaderImpl.log = log;
        autoLoaderImpl.recoverySrv = systemState;
        /*
         * Run Test Case 1
         */
        autoLoaderImpl.uninstall(fileToUninstall);
        verify(compState);
        verify(slState);
        verify(saState);
        verify(packageHandler);
        verify(autoLoaderImpl);
        verify(systemState);
    }
}
