/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DistributedJMXServerManagerImplTest.java 14:10:53 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.admin;

import java.lang.reflect.Method;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NameClassPair;
import javax.naming.NamingException;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.registry.mock.MockContext;
import org.objectweb.petals.kernel.admin.ContainerInformation.ContainerState;
import org.objectweb.petals.kernel.admin.util.ServerUtil;
import org.objectweb.petals.kernel.registry.jndi.NamingEnumerationImpl;
import org.objectweb.petals.util.InterfaceNamesHelper;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.Logger;

/**
 * Test of the DistributedJMXServerManagerImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class DistributedJMXServerManagerImplTest extends TestCase {

    private DistributedJMXServerFactoryImpl distributedJMXServerManagerImpl = new DistributedJMXServerFactoryImpl();

    public void setUp() throws Exception {
        distributedJMXServerManagerImpl.log = new LoggingUtil(Monolog
            .initialize().getLogger("test"));
    }

    public void testCreateDistributedJMXServer() throws Exception {
        ServerUtil.start(9999);
        MockContext containers = new MockContext();
        ContainerInformation containerInformation = new ContainerInformation();
        containerInformation.setHost("localhost");
        containerInformation.setJmxLogin("");
        containerInformation.setJmxPassword("");
        containerInformation.setName("0");
        containerInformation.setJmxPort("9999");
        containers.bind("0", containerInformation);
        distributedJMXServerManagerImpl.containersContext = containers;

        assertNotNull(distributedJMXServerManagerImpl
            .createDistributedJMXServer("0"));
        ServerUtil.stop();
    }

    public void testRetrieveContainerInformation() throws PetalsException {
        SystemUtil.setJmxPort("8081");
        ContainerInformation containerInformation = distributedJMXServerManagerImpl
            .retrieveContainerInformation();
        assertEquals(containerInformation.getJmxPort(), "8081");
    }

    public void testBindFc() throws NoSuchInterfaceException,
        IllegalBindingException, IllegalLifeCycleException {
        Logger logger = EasyMock.createMock(Logger.class);
        distributedJMXServerManagerImpl.bindFc(
            InterfaceNamesHelper.LOGGER_NAME, logger);
        assertNotNull(distributedJMXServerManagerImpl.log);
    }

    public void testBindFcException() throws NoSuchInterfaceException,
        IllegalBindingException, IllegalLifeCycleException {
        try {
            distributedJMXServerManagerImpl.bindFc("test", null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testGetFcState() throws IllegalLifeCycleException {
        assertEquals(LifeCycleController.STOPPED,
            distributedJMXServerManagerImpl.getFcState());
    }


    public void testLookupFc() {
        try {
            distributedJMXServerManagerImpl.lookupFc("test");
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testUnbindFc() {
        try {
            distributedJMXServerManagerImpl.unbindFc("test");
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testStartFc() throws SecurityException, NoSuchMethodException,
        IllegalLifeCycleException, PetalsException {
        DistributedJMXServerFactoryImpl distributedJMXServerFactoryImpl = EasyMock
            .createMock(DistributedJMXServerFactoryImpl.class, new Method[] {
                DistributedJMXServerFactoryImpl.class.getDeclaredMethod(
                    "initContext", new Class[0]),
                DistributedJMXServerFactoryImpl.class.getDeclaredMethod(
                    "retrieveContainerInformation", new Class[0]),
                DistributedJMXServerFactoryImpl.class.getDeclaredMethod(
                    "registerContainer", new Class[0])});
        ContainerInformation containerInformation = new ContainerInformation();

        EasyMock.expect(
            distributedJMXServerFactoryImpl.retrieveContainerInformation())
            .andReturn(containerInformation);
        distributedJMXServerFactoryImpl.registerContainer();
        distributedJMXServerFactoryImpl.initContext();

        EasyMock.replay(distributedJMXServerFactoryImpl);

        distributedJMXServerFactoryImpl.log = EasyMock
            .createMock(LoggingUtil.class);

        distributedJMXServerFactoryImpl.startFc();

        assertEquals(containerInformation,
            distributedJMXServerFactoryImpl.containerInformation);
    }

    public void testStartFcException() throws SecurityException,
        NoSuchMethodException, IllegalLifeCycleException, PetalsException {
        DistributedJMXServerFactoryImpl distributedJMXServerFactoryImpl = EasyMock
            .createMock(DistributedJMXServerFactoryImpl.class, new Method[] {
                DistributedJMXServerFactoryImpl.class.getDeclaredMethod(
                    "initContext", new Class[0]),
                DistributedJMXServerFactoryImpl.class.getDeclaredMethod(
                    "retrieveContainerInformation", new Class[0]),
                DistributedJMXServerFactoryImpl.class.getDeclaredMethod(
                    "registerContainer", new Class[0])});
        PetalsException petalsException = new PetalsException();
        LoggingUtil loggingUtil = EasyMock
        .createMock(LoggingUtil.class);
        
        loggingUtil.start();
        loggingUtil.end();
        loggingUtil.error(petalsException);
        EasyMock.expect(
            distributedJMXServerFactoryImpl.retrieveContainerInformation())
            .andThrow(petalsException);
        distributedJMXServerFactoryImpl.registerContainer();
        distributedJMXServerFactoryImpl.initContext();

        EasyMock.replay(distributedJMXServerFactoryImpl);
        EasyMock.replay(loggingUtil);

        distributedJMXServerFactoryImpl.log = loggingUtil;

        distributedJMXServerFactoryImpl.startFc();     
    }

    public void testIsContainerStarted() throws NamingException {
        DistributedJMXServerFactoryImpl distributedJMXServerFactoryImpl = new DistributedJMXServerFactoryImpl();
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        InitialContext initialContext = EasyMock
            .createMock(InitialContext.class);
        Context contContext = EasyMock.createMock(Context.class);
        ContainerInformation containerInformation = new ContainerInformation();
        containerInformation.setState(ContainerState.STARTED);

        loggingUtil.start();
        EasyMock.expect(initialContext.list("containers")).andReturn(
            new NamingEnumerationImpl<NameClassPair>(
                new NameClassPair[] {new NameClassPair("test", "foo")}));
        EasyMock.expect(contContext.lookup("test")).andReturn(
            containerInformation);
        loggingUtil.end();

        EasyMock.replay(loggingUtil);
        EasyMock.replay(contContext);
        EasyMock.replay(initialContext);

        distributedJMXServerFactoryImpl.containersContext = contContext;
        distributedJMXServerFactoryImpl.log = loggingUtil;
        distributedJMXServerFactoryImpl.rootContext = initialContext;

        assertTrue(distributedJMXServerFactoryImpl.isContainerStarted("test"));
    }
}
