/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JoramAgentImplTest.java 10:37:37 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.joram;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

import org.apache.commons.io.FileUtils;
import org.easymock.classextension.EasyMock;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.petals.kernel.admin.ContainerInformation;
import org.objectweb.petals.kernel.admin.ContainerInformation.ContainerState;
import org.objectweb.petals.util.InterfaceNamesHelper;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * Tests of the JoramAgentImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class JoramAgentImplTest extends TestCase {

    protected JoramAgentImpl agentImpl;

    protected String baseDir;

    public void setUp() {
        agentImpl = new JoramAgentImpl();
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        new File(baseDir
            + "target" + File.separator + "lib").mkdir();
        new File(baseDir
            + "target" + File.separator + "lib" + File.separator
            + "petals-1.0-SNAPSHOT.jar").mkdir();
        System.setProperty("petals.home", baseDir
            + "target");
    }

    public void testBindFcLookupFcUnbindFc() throws NoSuchInterfaceException,
        IllegalBindingException, IllegalLifeCycleException {
        Logger logger = EasyMock.createMock(Logger.class);
        agentImpl.bindFc(InterfaceNamesHelper.LOGGER_NAME, logger);
        assertNotNull(agentImpl.log);
        assertEquals(agentImpl.logger, logger);
        assertEquals(logger, agentImpl
            .lookupFc(InterfaceNamesHelper.LOGGER_NAME));
        agentImpl.unbindFc(InterfaceNamesHelper.LOGGER_NAME);
        assertNull(agentImpl.logger);
    }

    public void testBindFcException() {
        try {
            agentImpl.bindFc("foo", null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testUnbindFcException() {
        try {
            agentImpl.unbindFc("foo");
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testLookupFcException() {
        try {
            agentImpl.lookupFc("foo");
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testGetFcState() {
        agentImpl.fcState = LifeCycleController.STOPPED;
        assertEquals(agentImpl.getFcState(), LifeCycleController.STOPPED);
        agentImpl.fcState = LifeCycleController.STARTED;
        assertEquals(agentImpl.getFcState(), LifeCycleController.STARTED);
    }

    public void testListFc() {
        assertEquals(agentImpl.listFc().length, 0);
    }

    public void testInitConfiguration() throws IOException {
        agentImpl.initConfiguration();
        assertNotNull(agentImpl.factory);
    }

    public void testStartFc() throws Exception {
        JoramAgentImpl agentImpl = EasyMock.createMock(JoramAgentImpl.class,
            new Method[] {
                JoramAgentImpl.class.getDeclaredMethod("initJoramAgentServer",
                    new Class[] {}),
                JoramAgentImpl.class.getDeclaredMethod("startJoramAgentServer",
                    new Class[] {})});
        agentImpl.log = EasyMock.createMock(LoggingUtil.class);

        agentImpl.startJoramAgentServer();
        agentImpl.initJoramAgentServer();

        EasyMock.replay(agentImpl);

        agentImpl.startFc();
        assertEquals(agentImpl.fcState, LifeCycleController.STARTED);
    }

    public void testStopFc() throws IllegalLifeCycleException,
        SecurityException, NoSuchMethodException {
        JoramAgentImpl agentImpl = EasyMock.createMock(JoramAgentImpl.class,
            new Method[] {
                JoramAgentImpl.class.getDeclaredMethod("initJoramAgentServer",
                    new Class[] {}),
                JoramAgentImpl.class.getDeclaredMethod("startJoramAgentServer",
                    new Class[] {})});
        agentImpl.log = EasyMock.createMock(LoggingUtil.class);
        agentImpl.stopFc();
    }

    public void testStartServerDistributed() throws Exception {
        SystemUtil.setJoramId("0");
        SystemUtil.setHost("127.0.0.1");
        SystemUtil.setJoramTCPPort("16200");
        SystemUtil.setJoramDomainPort("16300");

        String conf = FileUtils.readFileToString(new File(baseDir
            + "src" + File.separator + "test-data" + File.separator + "joram"
            + File.separator + "joram-conf.xml"), "UTF-8");
        agentImpl = EasyMock.createMock(JoramAgentImpl.class, new Method[] {
            JoramAgentImpl.class.getDeclaredMethod(
                "retrieveDistributedConfiguration", new Class[0]),
            JoramAgentImpl.class.getDeclaredMethod("updateContainerState",
                new Class[] {String.class, ContainerState.class}),
            JoramAgentImpl.class.getDeclaredMethod("initJoramAgentServer",
                new Class[] {}),
            JoramAgentImpl.class.getDeclaredMethod("startJoramAgentServer",
                new Class[] {})});
        LoggingUtil log = EasyMock.createMock(LoggingUtil.class);

        EasyMock.expect(agentImpl.retrieveDistributedConfiguration())
            .andReturn(conf);

        log
            .debug("Problem while trying to retrieve all the containers information : context may not be initialized");
        log
            .info("Joram server started at [host:port 127.0.0.1:16300,tcp port 16200, jndi port , domain name eBM, server name 0, server id 0\n");

        log.start();
        log.start();
        log.end();
        agentImpl.initJoramAgentServer();
        agentImpl.startJoramAgentServer();

        EasyMock.replay(log);
        EasyMock.replay(agentImpl);

        agentImpl.log = log;

        agentImpl.startServer();
    }

    public void testStartServerException() throws SecurityException,
        NoSuchMethodException, IOException {
        SystemUtil.setJoramId("0");
        SystemUtil.setHost("127.0.0.1");
        SystemUtil.setJoramTCPPort("16200");
        SystemUtil.setJoramDomainPort("16300");

        agentImpl = EasyMock.createMock(JoramAgentImpl.class, new Method[] {
            JoramAgentImpl.class.getDeclaredMethod(
                "retrieveDistributedConfiguration", new Class[0]),
            JoramAgentImpl.class.getDeclaredMethod(
                "retrieveAllContainersInformation", new Class[0])});
        LoggingUtil log = EasyMock.createMock(LoggingUtil.class);

        RuntimeException runtimeException = new RuntimeException();

        EasyMock.expect(agentImpl.retrieveDistributedConfiguration()).andThrow(
            runtimeException).anyTimes();
        log.error("Problem while starting the Joram server", runtimeException);
        List<ContainerInformation> conts = new ArrayList<ContainerInformation>();
        conts.add(null);
        conts.add(null);
        EasyMock.expect(agentImpl.retrieveAllContainersInformation())
            .andReturn(conts);

        EasyMock.replay(log);
        EasyMock.replay(agentImpl);

        agentImpl.log = log;

        agentImpl.startServer();
    }

}
