/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MX4J_HtmlAdaptorTest.java 15:05:03 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.mx4j;

import junit.framework.TestCase;
import mx4j.tools.adaptor.http.HttpAdaptorMBean;

import org.easymock.classextension.EasyMock;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.jmx.agent.AdminAttributes;
import org.objectweb.petals.kernel.mx4j.mock.MockMBeanServer;
import org.objectweb.petals.util.InterfaceNamesHelper;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * Test the MX4JHtmlAdaptor
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class MX4JHtmlAdaptorTest extends TestCase {

    protected MX4JHtmlAdaptor adaptor;

    public void setUp() {
        adaptor = new MX4JHtmlAdaptor();
    }

    public void testGetters() {
        String state = adaptor.getFcState();
        if (!state.equals("STOPPED")) {
            fail();
        }
    }

    public void testBindFc() throws NoSuchInterfaceException {
        Logger logger = EasyMock.createMock(Logger.class);
        adaptor.bindFc(InterfaceNamesHelper.LOGGER_NAME, logger);
        assertEquals(adaptor.logger, logger);
        assertEquals(adaptor.lookupFc(InterfaceNamesHelper.LOGGER_NAME), logger);
        adaptor.unbindFc(InterfaceNamesHelper.LOGGER_NAME);
        assertNull(adaptor.logger);

        AdminAttributes adminAttributes = EasyMock
            .createMock(AdminAttributes.class);
        EasyMock.replay(adminAttributes);
        adaptor.bindFc(InterfaceNamesHelper.JMXAGENT_ADMINATT_NAME,
            adminAttributes);
        assertEquals(adaptor.adminAttributes, adminAttributes);
        assertEquals(adminAttributes, adaptor
            .lookupFc(InterfaceNamesHelper.JMXAGENT_ADMINATT_NAME));
        adaptor.unbindFc(InterfaceNamesHelper.JMXAGENT_ADMINATT_NAME);
        assertNull(adaptor.adminAttributes);
    }

    public void testBindFcException() {
        try {
            adaptor.bindFc("test", null);
            fail();
        } catch (NoSuchInterfaceException e) {
            // do nothing
        }
    }

    public void testLookupFcException() {
        try {
            adaptor.lookupFc("test");
            fail();
        } catch (NoSuchInterfaceException e) {
            // do nothing
        }
    }

    public void testUnbindFcException() {
        try {
            adaptor.unbindFc("test");
            fail();
        } catch (NoSuchInterfaceException e) {
            // do nothing
        }
    }

    public void testListFc() {
        assertEquals(adaptor.listFc()[0],
            InterfaceNamesHelper.JMXAGENT_ADMINATT_NAME);
    }

    public void testStartFcStopFc() {
        AdminAttributes adminAttributes = EasyMock
            .createMock(AdminAttributes.class);
        MockMBeanServer mockMBeanServer = new MockMBeanServer();
        EasyMock.expect(adminAttributes.getRawMBeanServer()).andReturn(
            mockMBeanServer).anyTimes();
        EasyMock.replay(adminAttributes);
        adaptor.adminAttributes = adminAttributes;
        adaptor.log = EasyMock.createMock(LoggingUtil.class);
        adaptor.startFc();
        assertTrue(mockMBeanServer.isRegisterMBean());
        assertEquals(adaptor.getFcState(), LifeCycleController.STARTED);
        adaptor.stopFc();
        assertEquals(adaptor.getFcState(), LifeCycleController.STOPPED);
        assertTrue(mockMBeanServer.isUnregisterMBean());
    }

    public void testStartFcAlreadyStarted() {
        SystemUtil.setHtmlPort("8082");

        HttpAdaptorMBean httpAdaptorMBean = EasyMock
            .createMock(HttpAdaptorMBean.class);

        EasyMock.expect(httpAdaptorMBean.isActive()).andReturn(Boolean.TRUE);
        EasyMock.expect(httpAdaptorMBean.getPort()).andReturn(8082);

        EasyMock.replay(httpAdaptorMBean);

        adaptor.adaptor = httpAdaptorMBean;
        adaptor.log = EasyMock.createMock(LoggingUtil.class);

        adaptor.startFc();
    }

    public void testStartFcStopAdaptor() {
        HttpAdaptorMBean httpAdaptorMBean = EasyMock
            .createMock(HttpAdaptorMBean.class);
        LoggingUtil log = EasyMock.createMock(LoggingUtil.class);
        AdminAttributes adminAttributes = EasyMock
            .createMock(AdminAttributes.class);
        RuntimeException runtimeException = new RuntimeException("test error");

        EasyMock.expect(httpAdaptorMBean.isActive()).andReturn(Boolean.FALSE);
        httpAdaptorMBean.stop();
        log.start();
        EasyMock.expect(adminAttributes.getRawMBeanServer()).andThrow(
            runtimeException);
        log.error("Fail to start HTML adaptor", runtimeException);
        log.end();

        EasyMock.replay(log);
        EasyMock.replay(httpAdaptorMBean);
        EasyMock.replay(adminAttributes);

        adaptor.adaptor = httpAdaptorMBean;
        adaptor.log = log;
        adaptor.adminAttributes = adminAttributes;

        adaptor.startFc();
    }

    public void testStopFcException() {
        AdminAttributes adminAttributes = EasyMock
            .createMock(AdminAttributes.class);
        LoggingUtil log = EasyMock.createMock(LoggingUtil.class);
        RuntimeException runtimeException = new RuntimeException(
            "test exception");

        log.start();
        EasyMock.expect(adminAttributes.getRawMBeanServer()).andThrow(
            runtimeException);
        log.warning("Error during MBean unregistering: test exception",
            runtimeException);
        log.end();

        EasyMock.replay(adminAttributes);
        EasyMock.replay(log);

        adaptor.adaptor = EasyMock.createMock(HttpAdaptorMBean.class);
        adaptor.log = log;
        adaptor.adminAttributes = adminAttributes;

        adaptor.stopFc();
    }

}
